# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tensorflow_datasets.core.tfrecords_writer."""

import json
import os
from unittest import mock

import tensorflow as tf
from tensorflow_datasets import testing
from tensorflow_datasets.core import dataset_utils
from tensorflow_datasets.core import example_parser
from tensorflow_datasets.core import example_serializer
from tensorflow_datasets.core import file_adapters
from tensorflow_datasets.core import lazy_imports_lib
from tensorflow_datasets.core import tfrecords_writer
from tensorflow_datasets.core import utils
from tensorflow_datasets.core.tfrecords_writer import _ShardSpec
from tensorflow_datasets.core.utils import shard_utils


class GetShardSpecsTest(testing.TestCase):
  # Here we don't need to test all possible reading configs, as this is tested
  # by shard_utils.py.

  @mock.patch.object(tfrecords_writer, '_get_number_shards',
                     mock.Mock(return_value=6))
  def test_1bucket_6shards(self):
    specs = tfrecords_writer._get_shard_specs(
        num_examples=8, total_size=16, bucket_lengths=[8], path='/bar.tfrecord')
    self.assertEqual(
        specs,
        [
            # Shard#, path, from_bucket, examples_number, reading instructions.
            _ShardSpec(0, '/bar.tfrecord-00000-of-00006',
                       '/bar.tfrecord-00000-of-00006_index.json', 1, [
                           shard_utils.FileInstruction(
                               filename='0', skip=0, take=1, num_examples=1),
                       ]),
            _ShardSpec(1, '/bar.tfrecord-00001-of-00006',
                       '/bar.tfrecord-00001-of-00006_index.json', 2, [
                           shard_utils.FileInstruction(
                               filename='0', skip=1, take=2, num_examples=2),
                       ]),
            _ShardSpec(2, '/bar.tfrecord-00002-of-00006',
                       '/bar.tfrecord-00002-of-00006_index.json', 1, [
                           shard_utils.FileInstruction(
                               filename='0', skip=3, take=1, num_examples=1),
                       ]),
            _ShardSpec(3, '/bar.tfrecord-00003-of-00006',
                       '/bar.tfrecord-00003-of-00006_index.json', 1, [
                           shard_utils.FileInstruction(
                               filename='0', skip=4, take=1, num_examples=1),
                       ]),
            _ShardSpec(4, '/bar.tfrecord-00004-of-00006',
                       '/bar.tfrecord-00004-of-00006_index.json', 2, [
                           shard_utils.FileInstruction(
                               filename='0', skip=5, take=2, num_examples=2),
                       ]),
            _ShardSpec(5, '/bar.tfrecord-00005-of-00006',
                       '/bar.tfrecord-00005-of-00006_index.json', 1, [
                           shard_utils.FileInstruction(
                               filename='0', skip=7, take=-1, num_examples=1),
                       ]),
        ])

  @mock.patch.object(tfrecords_writer, '_get_number_shards',
                     mock.Mock(return_value=2))
  def test_4buckets_2shards(self):
    specs = tfrecords_writer._get_shard_specs(
        num_examples=8,
        total_size=16,
        bucket_lengths=[2, 3, 0, 3],
        path='/bar.tfrecord')
    self.assertEqual(
        specs,
        [
            # Shard#, path, examples_number, reading instructions.
            _ShardSpec(0, '/bar.tfrecord-00000-of-00002',
                       '/bar.tfrecord-00000-of-00002_index.json', 4, [
                           shard_utils.FileInstruction(
                               filename='0', skip=0, take=-1, num_examples=2),
                           shard_utils.FileInstruction(
                               filename='1', skip=0, take=2, num_examples=2),
                       ]),
            _ShardSpec(1, '/bar.tfrecord-00001-of-00002',
                       '/bar.tfrecord-00001-of-00002_index.json', 4, [
                           shard_utils.FileInstruction(
                               filename='1', skip=2, take=-1, num_examples=1),
                           shard_utils.FileInstruction(
                               filename='3', skip=0, take=-1, num_examples=3),
                       ]),
        ])


class GetNumberShardsTest(testing.TestCase):

  def test_imagenet_train(self):
    size = 137 << 30  # 137 GiB
    num_examples = 1281167
    n = tfrecords_writer._get_number_shards(size, num_examples)
    self.assertEqual(n, 1024)

  def test_imagenet_evaluation(self):
    size = 6300 * (1 << 20)  # 6.3 GiB
    num_examples = 50000
    n = tfrecords_writer._get_number_shards(size, num_examples)
    self.assertEqual(n, 64)

  def test_verylarge_few_examples(self):
    size = 52 << 30  # 52 GiB
    num_examples = 512
    n = tfrecords_writer._get_number_shards(size, num_examples)
    self.assertEqual(n, 512)

  def test_xxl(self):
    size = 10 << 40  # 10 TiB
    num_examples = 10**9  # 1G
    n = tfrecords_writer._get_number_shards(size, num_examples)
    self.assertEqual(n, 11264)

  def test_xs(self):
    size = 100 << 20  # 100 MiB
    num_examples = 100 * 10**3  # 100K
    n = tfrecords_writer._get_number_shards(size, num_examples)
    self.assertEqual(n, 1)

  def test_m(self):
    size = 400 << 20  # 499 MiB
    num_examples = 200 * 10**3  # 200K
    n = tfrecords_writer._get_number_shards(size, num_examples)
    self.assertEqual(n, 4)


def _read_records(path, file_format=file_adapters.DEFAULT_FILE_FORMAT):
  """Returns (files_names, list_of_records_in_each_file).

  Args:
    path: path to tfrecord, omitting suffix.
    file_format: format of the record files.
  """
  # Ignore _index.json files.
  paths = sorted(tf.io.gfile.glob('%s-*-of-*' % path))
  paths = [
      p for p in paths if not p.endswith(tfrecords_writer._INDEX_PATH_SUFFIX)
  ]
  all_recs = []
  for fpath in paths:
    all_recs.append(
        list(
            dataset_utils.as_numpy(
                file_adapters.ADAPTER_FOR_FORMAT[file_format].make_tf_data(
                    fpath))))
  return [os.path.basename(p) for p in paths], all_recs


def _read_indices(path):
  """Returns (files_name, list of index in each file).

  Args:
    path: path to index, omitting suffix.
  """
  paths = sorted(tf.io.gfile.glob('%s-*-of-*_index.json' % path))
  all_indices = []
  for path in paths:
    json_str = utils.as_path(path).read_text()
    # parse it back into a proto.
    shard_index = json.loads(json_str)
    all_indices.append(list(shard_index['index']))
  return [os.path.basename(p) for p in paths], all_indices


class WriterTest(testing.TestCase):

  EMPTY_SPLIT_ERROR = 'No examples were yielded.'
  TOO_SMALL_SPLIT_ERROR = 'num_examples (1) < number_of_shards (2)'
  RECORDS_TO_WRITE = [
      (1, b'a'),
      (2, b'b'),
      (3, b'c'),
      (4, b'd'),
      (5, b'e'),
      (6, b'f'),
      (7, b'g'),
      (8, b'hi'),
  ]

  @mock.patch.object(example_serializer, 'ExampleSerializer',
                     testing.DummySerializer)
  def _write(self,
             to_write,
             path,
             salt='',
             disable_shuffling=False,
             file_format=file_adapters.DEFAULT_FILE_FORMAT):
    writer = tfrecords_writer.Writer(
        'some spec',
        path,
        hash_salt=salt,
        disable_shuffling=disable_shuffling,
        file_format=file_format)
    for key, record in to_write:
      writer.write(key, record)
    return writer.finalize()

  def test_write_tfrecord(self):
    """Writes 8 records in 5 shards.

    Number of records is evenly distributed (2-1-2-1-2).
    """
    path = os.path.join(self.tmp_dir, 'foo.tfrecord')
    with mock.patch.object(
        tfrecords_writer, '_get_number_shards', return_value=5):
      shards_length, total_size = self._write(self.RECORDS_TO_WRITE, path)
    self.assertEqual(shards_length, [2, 1, 2, 1, 2])
    self.assertEqual(total_size, 9)
    written_files, all_recs = _read_records(path)
    written_index_files, all_indices = _read_indices(path)
    self.assertEqual(written_files,
                     ['foo.tfrecord-0000%s-of-00005' % i for i in range(5)])
    self.assertEqual(all_recs, [
        [b'f', b'g'],
        [b'd'],
        [b'a', b'b'],
        [b'hi'],
        [b'e', b'c'],
    ])
    self.assertEmpty(written_index_files)
    self.assertEmpty(all_indices)

  def test_write_tfrecord_sorted_by_key(self):
    """Writes 8 records in 5 shards without shuffling.

    Number of records is evenly distributed (2-1-2-1-2).
    """
    path = os.path.join(self.tmp_dir, 'foo.tfrecord')
    with mock.patch.object(
        tfrecords_writer, '_get_number_shards', return_value=5):
      shards_length, total_size = self._write(
          self.RECORDS_TO_WRITE, path, disable_shuffling=True)
    self.assertEqual(shards_length, [2, 1, 2, 1, 2])
    self.assertEqual(total_size, 9)
    written_files, all_recs = _read_records(path)
    written_index_files, all_indices = _read_indices(path)
    self.assertEqual(written_files,
                     ['foo.tfrecord-0000%s-of-00005' % i for i in range(5)])
    self.assertEqual(all_recs, [
        [b'a', b'b'],
        [b'c'],
        [b'd', b'e'],
        [b'f'],
        [b'g', b'hi'],
    ])
    self.assertEmpty(written_index_files)
    self.assertEmpty(all_indices)

  @mock.patch.object(example_parser, 'ExampleParser', testing.DummyParser)
  def test_write_duplicated_keys(self):
    path = os.path.join(self.tmp_dir, 'foo.tfrecord')
    to_write = [(1, b'a'), (2, b'b'), (1, b'c')]
    with mock.patch.object(
        tfrecords_writer, '_get_number_shards', return_value=1):
      with self.assertRaisesWithPredicateMatch(
          AssertionError, 'Two examples share the same hashed key'):
        self._write(to_write, path)

  def test_empty_split(self):
    path = os.path.join(self.tmp_dir, 'foo.tfrecord')
    to_write = []
    with mock.patch.object(
        tfrecords_writer, '_get_number_shards', return_value=1):
      with self.assertRaisesWithPredicateMatch(AssertionError,
                                               self.EMPTY_SPLIT_ERROR):
        self._write(to_write, path)

  def test_too_small_split(self):
    path = os.path.join(self.tmp_dir, 'foo.tfrecord')
    to_write = [(1, b'a')]
    with mock.patch.object(
        tfrecords_writer, '_get_number_shards', return_value=2):
      with self.assertRaisesWithPredicateMatch(AssertionError,
                                               self.TOO_SMALL_SPLIT_ERROR):
        self._write(to_write, path)


class TfrecordsWriterBeamTest(WriterTest):

  EMPTY_SPLIT_ERROR = 'Not a single example present in the PCollection!'

  @mock.patch.object(example_serializer, 'ExampleSerializer',
                     testing.DummySerializer)
  def _write(self,
             to_write,
             path,
             salt='',
             disable_shuffling=False,
             file_format=file_adapters.DEFAULT_FILE_FORMAT):
    beam = lazy_imports_lib.lazy_imports.apache_beam
    writer = tfrecords_writer.BeamWriter(
        'some spec',
        path,
        salt,
        disable_shuffling=disable_shuffling,
        file_format=file_format,
    )
    # Here we need to disable type check as `beam.Create` is not capable of
    # inferring the type of the PCollection elements.
    options = beam.options.pipeline_options.PipelineOptions(
        pipeline_type_check=False)
    with beam.Pipeline(options=options) as pipeline:

      @beam.ptransform_fn
      def _build_pcollection(pipeline):
        pcollection = pipeline | 'Start' >> beam.Create(to_write)
        return writer.write_from_pcollection(pcollection)

      _ = pipeline | 'test' >> _build_pcollection()  # pylint: disable=no-value-for-parameter
    return writer.finalize()


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the Split API."""

from tensorflow_datasets import testing
from tensorflow_datasets.core import proto
from tensorflow_datasets.core import splits
from tensorflow_datasets.core.utils import shard_utils
import tensorflow_datasets.public_api as tfds

RANGE_TRAIN = list(range(0, 2000))
RANGE_TEST = list(range(3000, 3200))
RANGE_VAL = list(range(6000, 6010))


class SplitDictTest(testing.TestCase):

  def test_num_shards(self):
    si = tfds.core.SplitInfo(name="train", shard_lengths=[1, 2, 3], num_bytes=0)
    sd = splits.SplitDict([si], dataset_name="ds_name")
    self.assertEqual(sd["train"].num_shards, 3)

  def test_empty_split(self):
    sd = splits.SplitDict([], dataset_name="ds_name")
    with self.assertRaisesWithPredicateMatch(KeyError, "`splits` is empty"):
      _ = sd["train"]


class SplitsDictTest(testing.TestCase):

  @property
  def split_dict(self):
    si = [
        tfds.core.SplitInfo(name="train", shard_lengths=[10, 10], num_bytes=0),
        tfds.core.SplitInfo(name="test", shard_lengths=[1], num_bytes=0),
    ]
    sd = splits.SplitDict(si, dataset_name="ds_name")
    return sd

  def test_get(self):
    s = self.split_dict["train"]
    self.assertEqual("train", s.name)
    self.assertEqual(2, s.num_shards)

  def test_from_proto(self):
    sd = splits.SplitDict.from_proto(
        "ds_name",
        [proto.SplitInfo(name="validation", shard_lengths=[5], num_bytes=0)])
    self.assertIn("validation", sd)
    self.assertNotIn("train", sd)
    self.assertNotIn("test", sd)

  def test_to_proto(self):
    sd = self.split_dict
    sdp = sd.to_proto()

    # Split order is preserved
    self.assertEqual("train", sdp[0].name)
    self.assertEqual([10, 10], sdp[0].shard_lengths)

    self.assertEqual("test", sdp[1].name)
    self.assertEqual([1], sdp[1].shard_lengths)

  def test_bool(self):
    sd = splits.SplitDict([], dataset_name="ds_name")
    self.assertFalse(sd)  # Empty split is False
    si = [tfds.core.SplitInfo(name="train", shard_lengths=[5], num_bytes=0)]
    sd = splits.SplitDict(si, dataset_name="ds_name")
    self.assertTrue(sd)  # Non-empty split is True


class SplitsTest(testing.TestCase):

  @classmethod
  def setUpClass(cls):
    super(SplitsTest, cls).setUpClass()
    cls._builder = testing.DummyDatasetSharedGenerator(
        data_dir=testing.make_tmp_dir())
    cls._builder.download_and_prepare()

  def test_sub_split_num_examples(self):
    s = self._builder.info.splits
    self.assertEqual(s["train[75%:]"].num_examples, 5)
    self.assertEqual(s["train[:75%]"].num_examples, 15)
    self.assertEqual(
        s["train"].num_examples,
        s["train[75%:]"].num_examples + s["train[:75%]"].num_examples,
    )

    self.assertEqual(s["test[75%:]"].num_examples, 2)
    self.assertEqual(s["test[:75%]"].num_examples, 8)
    self.assertEqual(
        s["test"].num_examples,
        s["test[75%:]"].num_examples + s["test[:75%]"].num_examples,
    )

    self.assertEqual(s["all"].num_examples, s.total_num_examples)

  def test_sub_split_file_instructions(self):
    fi = self._builder.info.splits["train[75%:]"].file_instructions
    self.assertEqual(fi, [
        shard_utils.FileInstruction(
            filename="dummy_dataset_shared_generator-train.tfrecord-00000-of-00001",
            skip=15,
            take=-1,
            num_examples=5,
        )
    ])

  def test_sub_split_num_shards(self):
    self.assertEqual(self._builder.info.splits["train[75%:]"].num_shards, 1)

  def test_split_file_instructions(self):
    fi = self._builder.info.splits["train"].file_instructions
    self.assertEqual(fi, [
        shard_utils.FileInstruction(
            filename="dummy_dataset_shared_generator-train.tfrecord-00000-of-00001",
            skip=0,
            take=-1,
            num_examples=20,
        )
    ])

  def test_sub_split_filenames(self):
    self.assertEqual(self._builder.info.splits["train"].filenames, [
        "dummy_dataset_shared_generator-train.tfrecord-00000-of-00001",
    ])
    self.assertEqual(self._builder.info.splits["train[75%:]"].filenames, [
        "dummy_dataset_shared_generator-train.tfrecord-00000-of-00001",
    ])

  def test_sub_split_wrong_key(self):
    with self.assertRaisesWithPredicateMatch(ValueError,
                                             "Unknown split 'unknown'"):
      _ = self._builder.info.splits["unknown"]

  def test_split_enum(self):
    self.assertEqual(repr(splits.Split.TRAIN), "Split('train')")
    self.assertIsInstance(splits.Split.TRAIN, splits.Split)


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Util to add metadata into an existing dataset folder."""

import types
from typing import List, Union

from tensorflow_datasets.core import dataset_builder
from tensorflow_datasets.core import dataset_info
from tensorflow_datasets.core import features as features_lib
from tensorflow_datasets.core import file_adapters
from tensorflow_datasets.core import naming
from tensorflow_datasets.core import read_only_builder
from tensorflow_datasets.core import splits as split_lib
from tensorflow_datasets.core import utils
from tensorflow_datasets.core.folder_dataset import compute_split_utils
from tensorflow_datasets.core.utils import type_utils


class _WriteBuilder(
    dataset_builder.GeneratorBasedBuilder, skip_registration=True):
  """Dummy builder used as base to save metadata."""

  def _info(self):
    return dataset_info.DatasetInfo(builder=self)

  def _split_generators(self, dl_manager):
    return {}

  def _generate_examples(self):
    yield


def write_metadata(
    *,
    data_dir: type_utils.PathLike,
    features: features_lib.feature.FeatureConnectorArg,
    split_infos: Union[None, type_utils.PathLike,
                       List[split_lib.SplitInfo]] = None,
    version: Union[None, str, utils.Version] = None,
    check_data: bool = True,
    **ds_info_kwargs,
) -> None:
  """Add metadata required to load with TFDS.

  See documentation for usage:
  https://www.tensorflow.org/datasets/external_tfrecord

  Args:
    data_dir: Dataset path on which save the metadata
    features: dict of `tfds.features.FeatureConnector` matching the proto specs.
    split_infos: Can be either:  * A path to the pre-computed split info values
      ( the `out_dir` kwarg of `tfds.folder_dataset.compute_split_info`) * A
      list of `tfds.core.SplitInfo` (returned value of
      `tfds.folder_dataset.compute_split_info`) * `None` to auto-compute the
      split info.
    version: Optional dataset version (auto-infer by default, or fallback to
      1.0.0)
    check_data: If True, perform additional check to validate the data in
      data_dir is valid
    **ds_info_kwargs: Additional metadata forwarded to `tfds.core.DatasetInfo` (
      description, homepage,...). Will appear in the doc.
  """
  features = features_lib.features_dict.to_feature(features)
  data_dir = utils.as_path(data_dir)
  # Extract the tf-record filenames
  tfrecord_files = [
      f for f in data_dir.iterdir() if naming.FilenameInfo.is_valid(f.name)
  ]
  if not tfrecord_files:
    raise ValueError(
        f'Could not find tf-record (or compatible format) in {data_dir}. '
        'Make sure to follow the pattern: '
        '`<dataset_name>-<split_name>.<file-extension>-xxxxxx-of-yyyyyy`')

  file_infos = [naming.FilenameInfo.from_str(f.name) for f in tfrecord_files]

  # Use set with tuple expansion syntax to ensure all names are consistents
  snake_name, = {f.dataset_name for f in file_infos}
  camel_name = naming.snake_to_camelcase(snake_name)
  file_format, = {f.filetype_suffix for f in file_infos}
  file_format = file_adapters.file_format_from_suffix(file_format)

  cls = types.new_class(
      camel_name,
      bases=(_WriteBuilder,),
      kwds=dict(skip_registration=True),
      exec_body=None,
  )

  if version is None:  # Automatically detect the version
    if utils.Version.is_valid(data_dir.name):
      version = data_dir.name
    else:
      version = '1.0.0'
  cls.VERSION = utils.Version(version)

  # Create a dummy builder (use non existant folder to make sure
  # dataset_info.json is not restored)
  builder = cls(file_format=file_format, data_dir='/tmp/non-existent-dir/')

  # Create the metadata
  ds_info = dataset_info.DatasetInfo(
      builder=builder,
      features=features,
      **ds_info_kwargs,
  )
  ds_info.set_file_format(file_format)

  # Add the split infos
  split_dict = _load_splits(
      data_dir=data_dir,
      split_infos=split_infos,
      file_infos=file_infos,
      builder=builder,
  )
  ds_info.set_splits(split_dict)

  # Save all metadata (dataset_info.json, features.json,...)
  ds_info.write_to_directory(data_dir)

  # Make sure that the data can be loaded (feature connector match the actual
  # specs)
  if check_data:
    builder = read_only_builder.builder_from_directory(data_dir)
    split_name = next(iter(builder.info.splits))
    _, = builder.as_dataset(split=f'{split_name}[:1]')  # Load the first example


def _load_splits(
    *,
    data_dir: utils.ReadWritePath,
    split_infos: Union[None, type_utils.PathLike, List[split_lib.SplitInfo]],
    file_infos: List[naming.FilenameInfo],
    builder: dataset_builder.DatasetBuilder,
) -> split_lib.SplitDict:
  """Load the SplitDict which can be passed to DatasetInfo."""
  split_names = sorted(set(f.split for f in file_infos))

  if split_infos is None:  # Auto-compute the split-infos
    split_infos = compute_split_utils.compute_split_info(data_dir=data_dir)
  # Load the List[SplitInfo]
  elif isinstance(split_infos, type_utils.PathLikeCls):
    split_infos = compute_split_utils.split_infos_from_path(
        path=split_infos,
        split_names=split_names,
    )
  elif all(isinstance(s, split_lib.SplitInfo) for s in split_infos):
    given_splits = sorted([s.name for s in split_infos])
    if given_splits != split_names:
      raise ValueError(f'Splits {given_splits} do not match {split_names}')
  else:
    raise TypeError(f'Invalid split info: {split_infos}')

  split_protos = [s.to_proto() for s in split_infos]
  return split_lib.SplitDict.from_proto(
      repeated_split_infos=split_protos, dataset_name=builder.name)



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""smartwatch_gestures dataset."""

import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_DESCRIPTION = """
The **SmartWatch Gestures Dataset** has been collected to evaluate several gesture recognition algorithms for interacting with mobile applications using arm gestures.

Eight different users performed twenty repetitions of twenty different gestures, for a total of 3200 sequences.
Each sequence contains acceleration data from the 3-axis accelerometer of a first generation Sony SmartWatch™, as well as timestamps from the different clock sources available on an Android device.
The smartwatch was worn on the user's right wrist.
The gestures have been manually segmented by the users performing them by tapping the smartwatch screen at the beginning and at the end of every repetition.
"""

_CITATION = """
@INPROCEEDINGS{
  6952946,
  author={Costante, Gabriele and Porzi, Lorenzo and Lanz, Oswald and Valigi, Paolo and Ricci, Elisa},
  booktitle={2014 22nd European Signal Processing Conference (EUSIPCO)},
  title={Personalizing a smartwatch-based gesture interface with transfer learning},
  year={2014},
  volume={},
  number={},
  pages={2530-2534},
  doi={}}
"""


class SmartwatchGestures(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for smartwatch_gestures dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    class_label = tfds.features.ClassLabel(
        names_file=tfds.core.tfds_path(
            'time_series/smartwatch_gestures/class_labels.txt'))
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            'features':
                tfds.features.Sequence({
                    'time_millis': tf.uint64,
                    'time_nanos': tf.uint64,
                    'time_event': tf.uint64,
                    'accel_x': tf.float64,
                    'accel_y': tf.float64,
                    'accel_z': tf.float64,
                }),
            'participant':
                tf.uint8,
            'attempt':
                tf.uint8,
            'gesture':
                class_label
        }),
        supervised_keys=('features', 'gesture'),
        homepage='https://tev.fbk.eu/technologies/smartwatch-gestures-dataset',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    path = dl_manager.download_and_extract(
        'https://drive.google.com/uc?export=download&'
        'id=1nEs-JlAQv6xpuSIqahTKK68TgK37GirP')

    # There are no predefined train/val/test split for this dataset.
    # (smartwatch_gestures): Returns the Dict['train', Iterator[Key, Example]].
    return {
        'train': self._generate_examples(path / 'gestures-dataset'),
    }

  def _generate_examples(self, path):
    """Yields examples."""

    pd = tfds.core.lazy_imports.pandas

    for f in path.glob('*/*/*.txt'):

      table = pd.read_table(
          f,
          sep=' ',
          header=None,
          names=[
              'time_millis', 'time_nanos', 'time_event', 'accel_x', 'accel_y',
              'accel_z'
          ])

      # Create a unique key for each recorded gesture.
      participant_numstr = f.parent.parent.name[1:]  # Drop the "U".
      gesture_numstr = f.parent.name
      attempt_numstr = f.stem
      k = int(''.join([participant_numstr, gesture_numstr, attempt_numstr]))

      yield k, {
          'features': {
              'time_millis': table['time_millis'].to_numpy(),
              'time_nanos': table['time_nanos'].to_numpy(),
              'time_event': table['time_event'].to_numpy(),
              'accel_x': table['accel_x'].to_numpy(),
              'accel_y': table['accel_y'].to_numpy(),
              'accel_z': table['accel_z'].to_numpy(),
          },
          'participant': int(participant_numstr),
          'attempt': int(attempt_numstr),
          'gesture': int(gesture_numstr) - 1
      }



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Pathlib-like generic abstraction."""

import os
import typing
from typing import Callable, Dict, Tuple, Type, Union, TypeVar

from tensorflow_datasets.core.utils import gpath
from tensorflow_datasets.core.utils import type_utils

PathLike = type_utils.PathLike
ReadOnlyPath = type_utils.ReadOnlyPath
ReadWritePath = type_utils.ReadWritePath

PathLikeCls = Union[Type[ReadOnlyPath], Type[ReadWritePath]]
T = TypeVar('T')

_PATHLIKE_CLS: Tuple[PathLikeCls, ...] = (
    gpath.PosixGPath,
    gpath.WindowsGPath,
)
_URI_PREFIXES_TO_CLS: Dict[str, PathLikeCls] = {
    # Even on Windows, `gs://`,... are PosixPath
    uri_prefix: gpath.PosixGPath for uri_prefix in gpath.URI_PREFIXES
}


# pylint: disable=g-wrong-blank-lines
@typing.overload
def register_pathlike_cls(path_cls_or_uri_prefix: str) -> Callable[[T], T]:
  ...


@typing.overload
def register_pathlike_cls(path_cls_or_uri_prefix: T) -> T:
  ...


def register_pathlike_cls(path_cls_or_uri_prefix):
  """Register the class to be forwarded as-is in `as_path`.

  ```python
  @utils.register_pathlike_cls('my_path://')
  class MyPath(pathlib.PurePosixPath):
    ...

  my_path = tfds.core.as_path('my_path://some-path')
  ```

  Args:
    path_cls_or_uri_prefix: If a uri prefix is given, then passing calling
      `tfds.core.as_path('prefix://path')` will call the decorated class.

  Returns:
    The decorator or decoratorated class
  """
  global _PATHLIKE_CLS
  if isinstance(path_cls_or_uri_prefix, str):

    def register_pathlike_decorator(cls: T) -> T:
      _URI_PREFIXES_TO_CLS[path_cls_or_uri_prefix] = cls
      return register_pathlike_cls(cls)

    return register_pathlike_decorator
  else:
    _PATHLIKE_CLS = _PATHLIKE_CLS + (path_cls_or_uri_prefix,)
    return path_cls_or_uri_prefix


# pylint: enable=g-wrong-blank-lines


def as_path(path: PathLike) -> ReadWritePath:
  """Create a generic `pathlib.Path`-like abstraction.

  Depending on the input (e.g. `gs://`, `github://`, `ResourcePath`,...), the
  system (Windows, Linux,...), the function will create the right pathlib-like
  abstraction.

  Args:
    path: Pathlike object.

  Returns:
    path: The `pathlib.Path`-like abstraction.
  """
  is_windows = os.name == 'nt'
  if isinstance(path, str):
    uri_splits = path.split('://', maxsplit=1)
    if len(uri_splits) > 1:  # str is URI (e.g. `gs://`, `github://`,...)
      # On windows, `PosixGPath` is created for `gs://` paths
      return _URI_PREFIXES_TO_CLS[uri_splits[0] + '://'](path)  # pytype: disable=bad-return-type
    elif is_windows:
      return gpath.WindowsGPath(path)
    else:
      return gpath.PosixGPath(path)
  elif isinstance(path, _PATHLIKE_CLS):
    return path  # Forward resource path, gpath,... as-is  # pytype: disable=bad-return-type
  elif isinstance(path, os.PathLike):  # Other `os.fspath` compatible objects
    path_cls = gpath.WindowsGPath if is_windows else gpath.PosixGPath
    return path_cls(path)
  else:
    raise TypeError(f'Invalid path type: {path!r}')



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ogbg_molpcba dataset tests."""

from tensorflow_datasets.graphs.ogbg_molpcba import ogbg_molpcba
import tensorflow_datasets.public_api as tfds


class OgbgMolpcbaTest(tfds.testing.DatasetBuilderTestCase):
  """Tests for ogbg_molpcba dataset."""
  # TODO(ameyasd): Add reasonable tests.
  DATASET_CLASS = ogbg_molpcba.OgbgMolpcba
  SPLITS = {
      'train': 4,  # Number of fake train examples.
      'validation': 4,  # Number of fake validation examples.
      'test': 4,  # Number of fake test examples.
  }

if __name__ == '__main__':
  tfds.testing.test_main()



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Imagenet validation with multi-label annotations (http://proceedings.mlr.press/v119/shankar20c.html)."""

import json
import os
import tarfile

import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_DESCRIPTION = """\
This dataset contains ILSVRC-2012 (ImageNet) validation images annotated with
multi-class labels from
["Evaluating Machine Accuracy on ImageNet"](http://proceedings.mlr.press/v119/shankar20c/shankar20c.pdf),
ICML, 2020.  The multi-class labels were reviewed by a panel of experts
extensively trained in the intricacies of fine-grained class distinctions in the
ImageNet class hierarchy (see paper for more details).
Compared to the original labels, these expert-reviewed multi-class labels enable
a more semantically coherent evaluation of accuracy.

Only 20,000 of the 50,000 ImageNet validation images have multi-label
annotations.  The set of multi-labels was first generated by a testbed of 67
trained ImageNet models, and then each individual model prediction was manually
annotated by the experts as either `correct` (the label is correct for the
image),`wrong` (the label is incorrect for the image), or `unclear` (no
consensus was reached among the experts).

Additionally, during annotation, the expert panel identified a set of
*problematic images*. An image was problematic if it met any of the below
criteria:

  * The original ImageNet label (top-1 label) was incorrect or unclear
  * Image was a drawing, painting, sketch, cartoon, or computer-rendered
  * Image was excessively edited
  * Image had inappropriate content

The problematic images are included in this dataset but should be ignored when
computing multi-label accuracy. Additionally, since the initial set of 20,000
annotations is class-balanced, but the set of problematic images is not, we
recommend computing the per-class accuracies and then averaging them. We also
recommend counting a prediction as correct if it is marked as correct or unclear
(i.e., being lenient with the unclear labels).

One possible way of doing this is with the following NumPy code:

```python
import tensorflow_datasets as tfds

ds = tfds.load('imagenet2012_multilabel', split='validation')

# We assume that predictions is a dictionary from file_name to a class index between 0 and 999

num_correct_per_class = {}
num_images_per_class = {}

for example in ds:
    # We ignore all problematic images
    if example[‘is_problematic’].numpy():
        continue

    # The label of the image in ImageNet
    cur_class = example['original_label'].numpy()

    # If we haven't processed this class yet, set the counters to 0
    if cur_class not in num_correct_per_class:
        num_correct_per_class[cur_class] = 0
        assert cur_class not in num_images_per_class
        num_images_per_class[cur_class] = 0

    num_images_per_class[cur_class] += 1

    # Get the predictions for this image
    cur_pred = predictions[example['file_name'].numpy()]

    # We count a prediction as correct if it is marked as correct or unclear
    # (i.e., we are lenient with the unclear labels)
    if cur_pred is in example['correct_multi_labels'].numpy() or cur_pred is in example['unclear_multi_labels'].numpy():
        num_correct_per_class[cur_class] += 1

# Check that we have collected accuracy data for each of the 1,000 classes
num_classes = 1000
assert len(num_correct_per_class) == num_classes
assert len(num_images_per_class) == num_classes

# Compute the per-class accuracies and then average them
final_avg = 0
for cid in range(num_classes):
  assert cid in num_correct_per_class
  assert cid in num_images_per_class
  final_avg += num_correct_per_class[cid] / num_images_per_class[cid]
final_avg /= num_classes

```
"""

_CITATION = """\
@article{shankar2019evaluating,
  title={Evaluating Machine Accuracy on ImageNet},
  author={Vaishaal Shankar* and Rebecca Roelofs* and Horia Mania and Alex Fang and Benjamin Recht and Ludwig Schmidt},
  journal={ICML},
  year={2020},
  note={\\url{http://proceedings.mlr.press/v119/shankar20c.html}}
}
@article{ImageNetChallenge,
  title={{ImageNet} large scale visual recognition challenge},
  author={Olga Russakovsky and Jia Deng and Hao Su and Jonathan Krause
   and Sanjeev Satheesh and Sean Ma and Zhiheng Huang and Andrej Karpathy and Aditya Khosla and Michael Bernstein and
   Alexander C. Berg and Fei-Fei Li},
  journal={International Journal of Computer Vision},
  year={2015},
  note={\\url{https://arxiv.org/abs/1409.0575}}
}
@inproceedings{ImageNet,
   author={Jia Deng and Wei Dong and Richard Socher and Li-Jia Li and Kai Li and Li Fei-Fei},
   booktitle={Conference on Computer Vision and Pattern Recognition (CVPR)},
   title={{ImageNet}: A large-scale hierarchical image database},
   year={2009},
   note={\\url{http://www.image-net.org/papers/imagenet_cvpr09.pdf}}
}
"""

_VALIDATION_LABELS_FNAME = 'image_classification/imagenet2012_validation_labels.txt'
_LABELS_FNAME = 'image_classification/imagenet2012_labels.txt'

_MULTI_LABELS_URL = 'https://imagenetv2public.s3-us-west-2.amazonaws.com/human_accuracy_fixed.json'


def _get_multi_labels_and_problematic_images(
    dl_manager: tfds.download.DownloadManager):
  """Returns multi-labels and problematic images from download json.

  Args:
    dl_manager: tfds.download.DownloadManager for downloading the json file

  Returns:
    val_annotated_images: Dictionary mapping image name to an inner dictionary
      containing the multi_label annotations for that image. The inner multi-
      label annotation dictionary has keys 'correct', 'wrong', or 'unclear'
      (keys will be missing if the image does not have a set of labels of the
      given type) and values that are lists of wnids.
    problematic_images: List of image names for problematic images.
  """
  with tf.io.gfile.GFile(dl_manager.download(_MULTI_LABELS_URL), 'r') as f:
    human_accuracy_data = json.load(f)
  val_annotated_images = {}
  prefix = 'ILSVRC2012_val_'
  len_prefix = len(prefix)
  for image_name in human_accuracy_data['initial_annots'].keys():
    if image_name[:len_prefix] == prefix:
      val_annotated_images[image_name] = human_accuracy_data['initial_annots'][
          image_name]

  problematic_images = list(human_accuracy_data['problematic_images'].keys())
  return val_annotated_images, problematic_images


def _get_original_labels(val_path):
  """Returns original labels for ImageNet validation data.

  Args:
    val_path: path to TAR file containing validation images. It is used to
      retrieve the name of pictures and associate them to labels.

  Returns:
    dict, mapping from image name (str) to label (str).
  """
  labels_path = os.fspath(tfds.core.tfds_path(_VALIDATION_LABELS_FNAME))
  with tf.io.gfile.GFile(labels_path) as labels_f:
    # `splitlines` to remove trailing `\r` in Windows
    labels = labels_f.read().strip().splitlines()
  with tf.io.gfile.GFile(val_path, 'rb') as tar_f_obj:
    tar = tarfile.open(mode='r:', fileobj=tar_f_obj)
    images = sorted(tar.getnames())
  return dict(zip(images, labels))


class Imagenet2012Multilabel(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for imagenet2012_multilabel dataset."""

  VERSION = tfds.core.Version('2.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
      '2.0.0': 'Fixed ILSVRC2012_img_val.tar file.',
  }

  MANUAL_DOWNLOAD_INSTRUCTIONS = """\
  manual_dir should contain `ILSVRC2012_img_val.tar` file.
  You need to register on http://www.image-net.org/download-images in order
  to get the link to download the dataset.
  """

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    names_file = tfds.core.tfds_path(_LABELS_FNAME)
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            'image':
                tfds.features.Image(encoding_format='jpeg'),
            'original_label':
                tfds.features.ClassLabel(names_file=names_file),
            'correct_multi_labels':
                tfds.features.Sequence(
                    tfds.features.ClassLabel(names_file=names_file)),
            'wrong_multi_labels':
                tfds.features.Sequence(
                    tfds.features.ClassLabel(names_file=names_file)),
            'unclear_multi_labels':
                tfds.features.Sequence(
                    tfds.features.ClassLabel(names_file=names_file)),
            'is_problematic':
                tfds.features.Tensor(shape=(), dtype=tf.bool),
            'file_name':
                tfds.features.Text(),
        }),
        supervised_keys=('image', 'correct_multi_labels'),
        homepage='https://github.com/modestyachts/evaluating_machine_accuracy_on_imagenet',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    val_path = os.path.join(dl_manager.manual_dir, 'ILSVRC2012_img_val.tar')
    if not tf.io.gfile.exists(val_path):
      raise AssertionError(
          'ImageNet requires manual download of the data. Please download '
          'the train and val set and place them into: {}'.format(val_path))
    multi_and_prob = _get_multi_labels_and_problematic_images(dl_manager)
    return {
        'validation':
            self._generate_examples(
                archive=dl_manager.iter_archive(val_path),
                original_labels=_get_original_labels(val_path),
                multi_labels=multi_and_prob[0],
                problematic_images=multi_and_prob[1])
    }

  def _generate_examples(self, archive, original_labels, multi_labels,
                         problematic_images):
    """Yields (key, example) tuples from the dataset."""
    for fname, fobj in archive:
      if fname not in multi_labels:
        # Image is not in the annotated set of 20,000 images
        continue
      else:
        is_problematic = fname in problematic_images
        correct_multi_labels = []
        wrong_multi_labels = []
        unclear_multi_labels = []
        if 'correct' in multi_labels[fname]:
          correct_multi_labels = multi_labels[fname]['correct']
        if 'wrong' in multi_labels[fname]:
          wrong_multi_labels = multi_labels[fname]['wrong']
        if 'unclear' in multi_labels[fname]:
          unclear_multi_labels = multi_labels[fname]['unclear']

      record = {
          'file_name': fname,
          'image': fobj,
          'original_label': original_labels[fname],
          'correct_multi_labels': correct_multi_labels,
          'wrong_multi_labels': wrong_multi_labels,
          'unclear_multi_labels': unclear_multi_labels,
          'is_problematic': is_problematic
      }
      yield fname, record



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""*-CFQ (Star-CFQ) dataset suite."""

import json
import os
from absl import logging
import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_CITATION = """
@inproceedings{Tsarkov2021,
  title={*-CFQ: Analyzing the Scalability of Machine Learning on a Compositional Task},
  author={Dmitry Tsarkov and Tibor Tihon and Nathan Scales and Nikola Momchev and Danila Sinopalnikov and Nathanael Sch\"{a}rli},
  booktitle={AAAI},
  year={2021},
  url={https://arxiv.org/abs/2012.08266},
}
"""

_DESCRIPTION = """
The *-CFQ datasets (and their splits) for measuring the scalability of
compositional generalization.

See https://arxiv.org/abs/2012.08266 for background.

Example usage:

```
data = tfds.load('star_cfq/single_pool_10x_b_cfq')
```
"""

_DATA_URL = 'https://storage.googleapis.com/star_cfq_dataset'

_RANDOM_SEEDS = [
    '4_0',
    '4_42',
    '4_81',
    '4_86',
    '4.25_0',
    '4.25_121',
    '4.25_125',
    '4.25_142',
    '4.5_0',
    '4.5_45',
    '4.5_82',
    '4.5_87',
    '4.75_0',
    '4.75_122',
    '4.75_126',
    '4.75_145',
    '5_0',
    '5_50',
    '5_83',
    '5_88',
    '5.25_0',
    '5.25_123',
    '5.25_127',
    '5.25_150',
    '5.5_0',
    '5.5_55',
    '5.5_84',
    '5.5_89',
    '5.75_0',
    '5.75_124',
    '5.75_128',
    '5.75_155',
    '6_0',
    '6_71',
    '6_85',
    '6_90',
]

_SHARD_SIZE = 95742

# '' is used for CFQ, 'o' is used for o-CFQ, and so on.
_POOLS = ['', 'o', 'u', 'b', 'l', 'half_l', 'n', 'half_n', 'x', 'half_x']
_SUPPLEMENTARY_POOLS = ['l', 'half_l', 'n', 'half_n', 'x', 'half_x']

_UCFQ_POOL = 'u-CFQ'

_NONBLENDED_TRAIN_SET_SIZES = {
    'default': [0.25, 0.5, 0.75, 1, 3, 10, 30, 80, 100],
    '': [0.1, 0.2, 0.3, 0.5, 1, 2],
    'b': [
        0.1, 0.2, 0.25, 0.3, 0.4, 0.5, 0.6, 0.7, 0.75, 0.8, 1, 1.5, 2, 3, 4, 5,
        6, 8, 10, 11, 12, 13, 15, 30, 35, 80, 85, 100
    ],
    'o': [0.1, 0.2, 0.3, 0.5, 1, 2],
    'u': [0.1, 0.2, 0.3, 0.5, 1, 2, 3, 6, 10, 30, 100]
}
_BLENDING_INITIAL_SIZES = [0, 0.1, 0.2, 0.3, 0.5, 1, 2, 3, 5]
_BLENDING_SUPPLEMENTARY_SIZES = [0.1, 0.3, 1, 2, 3, 10, 30, 80, 100]
_UNIQUE_INITIAL_SIZES = [0.1, 1]
_OVERWEIGHTING_SIZES = [0.1, 0.3, 1, 3, 10, 30, 80]
# Smaller sizes are skipped to limit the number of generated builder configs.
_DIVERGENCE_SPLIT_SIZES = [0.333333]

_QUESTION = 'question'
_QUERY = 'query'
_QUESTION_FIELD = 'questionPatternModEntities'
_QUERY_FIELD = 'sparqlPatternModEntities'


class StarCFQConfig(tfds.core.BuilderConfig):
  """BuilderConfig for *-CFQ splits."""

  def __init__(self,
               *,
               name,
               split_archive_path,
               split_path,
               compound_divergence=False,
               **kwargs):
    """BuilderConfig for a *-CFQ dataset.

    Args:
      name: Unique name of the split.
      split_archive_path: Path to the archive containing the split file.
      split_path: Relative path to the split file in the archive.
      compound_divergence: If true, the config corresponds to a compound
        divergence split. Otherwise, it corresponds to a random split.
      **kwargs: keyword arguments forwarded to super.
    """
    super(StarCFQConfig, self).__init__(
        name=name, version=tfds.core.Version('1.1.0'), **kwargs)
    self.split_archive_path = split_archive_path
    self.split_path = split_path
    self.compound_divergence = compound_divergence


def _generate_compound_divergence_builder_configs():
  """Generate configs for different compound divergences and random seeds."""
  configs = []
  for size in _DIVERGENCE_SPLIT_SIZES:
    for compound_divergence in [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6]:
      for seed_idx, seed in enumerate(_RANDOM_SEEDS):
        configs.append(
            StarCFQConfig(
                name=f'u_cfq_compound_divergence_{size}_{compound_divergence}_r{seed_idx}',
                compound_divergence=True,
                split_archive_path='splits/u-cfq-divergence-splits-1.0.tar.gz',
                split_path=f'divergence_splits/divergence_split_s{size}_d{compound_divergence}_r{seed}.json'
            ))

  return configs


def _generate_single_pool_random_split_builder_configs():
  """Generate configs for random splits using the same pool for train & test."""
  configs = []
  for pool in _POOLS:
    for size in _NONBLENDED_TRAIN_SET_SIZES.get(
        pool, _NONBLENDED_TRAIN_SET_SIZES['default']):
      full_pool_name = '_'.join([pool, 'cfq']).lstrip('_')
      configs.append(
          StarCFQConfig(
              name=f'single_pool_{size}x_{full_pool_name}',
              split_archive_path='splits/star-cfq-random-splits-1.0.tar.gz',
              split_path=f'random_splits/train_{size}x_{full_pool_name}__test_1x_{full_pool_name}.json'
          ))
  return configs


def _generate_ungrounded_on_grounded_builder_configs():
  """Generate configs for random splits training on ungrounded and testing on grounded data."""
  configs = []
  for size in _NONBLENDED_TRAIN_SET_SIZES['u']:
    # Out of the usual train set sizes for u-CFQ, there was no split created
    # with 6x u-CFQ train set and an o-CFQ test set.
    if size == 6:
      continue
    configs.append(
        StarCFQConfig(
            name=f'ungrounded_on_grounded_{size}x',
            split_archive_path='splits/star-cfq-random-splits-1.0.tar.gz',
            split_path=f'random_splits/train_{size}x_u_cfq__test_1x_o_cfq.json')
    )
  return configs


def _generate_blended_split_builder_configs():
  """Generate configs for blended random splits."""
  configs = []
  split_archive_path = 'splits/star-cfq-random-splits-1.0.tar.gz'
  for supplementary_pool in _SUPPLEMENTARY_POOLS:
    for initial_size in _BLENDING_INITIAL_SIZES:
      for supplementary_size in _BLENDING_SUPPLEMENTARY_SIZES:
        # Some splits with 80x n-CFQ/half-n-CFQ sizes are missing, skip them.
        if supplementary_size == 80 and (supplementary_pool in ['n', 'half_n']
                                         or initial_size not in [0, 0.1, 1]):
          continue
        if initial_size != 0:
          configs.append(
              StarCFQConfig(
                  name=f'equal_weighting_{initial_size}x_b_cfq_{supplementary_size}x_{supplementary_pool}_cfq',
                  split_archive_path=split_archive_path,
                  split_path=f'random_splits/train_{initial_size}x_b_cfq_{supplementary_size}x_{supplementary_pool}_cfq__test_1x_b_cfq.json'
              ))
        else:
          configs.append(
              StarCFQConfig(
                  name=f'equal_weighting_0x_b_cfq_{supplementary_size}x_{supplementary_pool}_cfq',
                  split_archive_path=split_archive_path,
                  split_path=f'random_splits/train_{supplementary_size}x_{supplementary_pool}_cfq__test_1x_b_cfq.json'
              ))
  for unique_initial_size in _UNIQUE_INITIAL_SIZES:
    for supplementary_pool in _SUPPLEMENTARY_POOLS:
      for size in _OVERWEIGHTING_SIZES:
        # Splits with 80x n-CFQ/half-n-CFQ sizes are missing, skip them for now.
        if size == 80 and supplementary_pool in ['n', 'half_n']:
          continue
        # This split file is missing, skip it in this version.
        if unique_initial_size == 1 and size == 1 and supplementary_pool == 'half_n':
          continue
        configs.append(
            StarCFQConfig(
                name=f'overweighting_{unique_initial_size}x_unique_{size}x_b_cfq_{size}x_{supplementary_pool}_cfq',
                split_archive_path=split_archive_path,
                split_path=f'random_splits/train_{unique_initial_size}x_unique_{size}x_b_cfq_{size}x_{supplementary_pool}_cfq__test_1x_b_cfq.json'
            ))

  return configs


def _read_pool_shard(pool_path, index):
  path = os.path.join(pool_path, 'dataset_%03d.json' % index)
  logging.info('Reading %s...', path)
  with tf.io.gfile.GFile(path) as f:
    questions = json.load(f)
  return questions


class Slice:

  def __init__(self, shard_index, range_begin, range_end):
    self.shard_index = shard_index
    self.range_begin = range_begin
    self.range_end = range_end


def _get_dataset_slices(start, end):
  """Splits an absolute range to slices split by shard."""
  slices = []
  for shard_idx in range(int((end - 1) / _SHARD_SIZE) + 1):
    start_offset = shard_idx * _SHARD_SIZE
    end_offset = (shard_idx + 1) * _SHARD_SIZE
    if end_offset <= start:
      continue
    s = Slice(shard_idx,
              max(start, start_offset) % _SHARD_SIZE,
              (min(end, end_offset) - 1) % _SHARD_SIZE + 1)
    slices.append(s)
  return slices


class StarCFQ(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for a *-CFQ split."""

  BUILDER_CONFIGS = _generate_single_pool_random_split_builder_configs(
  ) + _generate_blended_split_builder_configs(
  ) + _generate_ungrounded_on_grounded_builder_configs(
  ) + _generate_compound_divergence_builder_configs()

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            _QUESTION: tfds.features.Text(),
            _QUERY: tfds.features.Text(),
        }),
        supervised_keys=(_QUESTION, _QUERY),
        homepage='https://github.com/google-research/google-research/tree/master/star-cfq',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    split_dir = dl_manager.download_and_extract(
        '%s/%s' % (_DATA_URL, self.builder_config.split_archive_path))
    split_path = os.path.join(split_dir, self.builder_config.split_path)

    dataset_paths = {}
    if self.builder_config.compound_divergence:
      extracted_dataset_path = dl_manager.download_and_extract(
          _DATA_URL +
          '/datasets/u-cfq-for-divergence-splits-1.0-compact-combined.tar.gz')
      dataset_paths[_UCFQ_POOL] = os.path.join(
          extracted_dataset_path,
          'u-cfq-for-divergence-splits-1.0-compact-combined')

    else:
      with tf.io.gfile.GFile(split_path) as file:
        splits = json.load(file)
        for split in splits.values():
          for s in split:
            name = s['dataset']
            if name in dataset_paths:
              continue
            logging.info('Downloading dataset %s...', name)
            extracted_dataset_path = dl_manager.download_and_extract(
                _DATA_URL + f'/datasets/{name.lower()}-1.0-compact.tar.gz')
            dataset_paths[name] = os.path.join(extracted_dataset_path,
                                               f'{name.lower()}-1.0-compact')

    if self.builder_config.compound_divergence:
      return {
          'train':
              self._generate_examples(dataset_paths, split_path, 'trainIdxs'),
          'validation':
              self._generate_examples(dataset_paths, split_path, 'devIdxs'),
          'test':
              self._generate_examples(dataset_paths, split_path, 'testIdxs')
      }
    return {
        'train': self._generate_examples(dataset_paths, split_path, 'train'),
        'test': self._generate_examples(dataset_paths, split_path, 'test')
    }

  def _generate_examples(self, dataset_paths, split_path, split_id):
    """Yields examples."""
    if self.builder_config.compound_divergence:
      samples_path = os.path.join(dataset_paths[_UCFQ_POOL], 'dataset.json')
      with tf.io.gfile.GFile(samples_path) as samples_file:
        logging.info('Reading json from %s into memory...', samples_path)
        samples = json.loads(samples_file.read())
        logging.info('%d samples loaded', len(samples))
        with tf.io.gfile.GFile(split_path) as split_file:
          splits = json.loads(split_file.read())
          for idx in splits[split_id]:
            sample = samples[idx]
            yield idx, {
                _QUESTION: sample[_QUESTION_FIELD],
                _QUERY: sample[_QUERY_FIELD]
            }
    else:
      with tf.io.gfile.GFile(split_path) as sf:
        split_config = json.load(sf)
      slice_index = 0
      for dss in split_config[split_id]:
        previous_shard = -1
        pool = None
        dataset = dss['dataset']
        if dataset not in dataset_paths:
          logging.error('Unexpected dataset %s', dataset)
          continue
        pool_path = dataset_paths[dataset]
        for rng in dss['indices']:
          start = int(rng['startIndex'])
          end = int(rng['endIndex'])
          for s in _get_dataset_slices(start, end):
            if previous_shard != s.shard_index:
              pool = _read_pool_shard(pool_path, s.shard_index)
              previous_shard = s.shard_index
            for idx in range(s.range_begin, s.range_end):
              absolute_index = s.shard_index * _SHARD_SIZE + idx
              # The slice_index is also part of the key because overweighting
              # splits use the same example multiple times and the key must
              # be unique for each instance.
              key = f'{dataset}-{absolute_index}-{slice_index}'
              yield key, {
                  _QUESTION: pool[idx][_QUESTION_FIELD],
                  _QUERY: pool[idx][_QUERY_FIELD],
              }
            slice_index += 1



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tensorflow_datasets.core.load.

Note: `load.py` code was previously in `registered.py`, so some of the tests
are still on `registered_test.py`.
"""

from unittest import mock

import pytest
from tensorflow_datasets import testing
from tensorflow_datasets.core import load
from tensorflow_datasets.core import read_only_builder
from tensorflow_datasets.core import registered
from tensorflow_datasets.core import utils
from tensorflow_datasets.core import visibility


@visibility.set_availables_tmp([
    visibility.DatasetType.COMMUNITY_PUBLIC,
])
def test_community_public_load():
  with mock.patch(
      'tensorflow_datasets.core.community.community_register.list_builders',
      return_value=['ns:ds'],
  ), mock.patch(
      'tensorflow_datasets.core.community.community_register.builder_cls',
      return_value=testing.DummyDataset,
  ):
    assert load.list_builders() == ['ns:ds']

    # Builder is correctly returned
    assert load.builder_cls('ns:ds') is testing.DummyDataset
    assert isinstance(load.builder('ns:ds'), testing.DummyDataset)



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Largest Covid-19 epidemiological database with a powerful set of expansive covariates.

It includes open, publicly sourced, licensed data relating to demographics,
economy, epidemiology, geography, health, hospitalizations, mobility, government
response, weather, and more.
"""

import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_DESCRIPTION = """
This repository attempts to assemble the largest Covid-19 epidemiological database
in addition to a powerful set of expansive covariates. It includes open, publicly sourced,
licensed data relating to demographics, economy, epidemiology, geography, health, hospitalizations,
mobility, government response, weather, and more.

This particular dataset corresponds to a join of all the different tables that are part of the repository.
Therefore, expect the resulting samples to be highly sparse.
"""

_CITATION = """
@article{Wahltinez2020,
  author = "O. Wahltinez and others",
  year = 2020,
  title = "COVID-19 Open-Data: curating a fine-grained, global-scale data repository for SARS-CoV-2",
  note = "Work in progress",
  url = {https://goo.gle/covid-19-open-data},
}
"""

_N_RECORDS = 11266423  # It should be an upper bound
_BATCH_SIZE = 10000


class Covid19(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for covid19 dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            'location_key':
                tf.string,
            'date':
                tf.string,
            'place_id':
                tf.string,
            'wikidata_id':
                tf.string,
            'datacommons_id':
                tf.string,
            'country_code':
                tf.string,
            'country_name':
                tf.string,
            'iso_3166_1_alpha_2':
                tf.string,
            'iso_3166_1_alpha_3':
                tf.string,
            'aggregation_level':
                tf.float64,
            'new_confirmed':
                tf.float64,
            'new_deceased':
                tf.float64,
            'cumulative_confirmed':
                tf.float64,
            'cumulative_deceased':
                tf.float64,
            'cumulative_tested':
                tf.float64,
            'new_persons_vaccinated':
                tf.float64,
            'cumulative_persons_vaccinated':
                tf.float64,
            'new_persons_fully_vaccinated':
                tf.float64,
            'cumulative_persons_fully_vaccinated':
                tf.float64,
            'new_vaccine_doses_administered':
                tf.float64,
            'cumulative_vaccine_doses_administered':
                tf.float64,
            'population':
                tf.float64,
            'population_male':
                tf.float64,
            'population_female':
                tf.float64,
            'population_rural':
                tf.float64,
            'population_urban':
                tf.float64,
            'population_density':
                tf.float64,
            'human_development_index':
                tf.float64,
            'population_age_00_09':
                tf.float64,
            'population_age_10_19':
                tf.float64,
            'population_age_20_29':
                tf.float64,
            'population_age_30_39':
                tf.float64,
            'population_age_40_49':
                tf.float64,
            'population_age_50_59':
                tf.float64,
            'population_age_60_69':
                tf.float64,
            'population_age_70_79':
                tf.float64,
            'population_age_80_and_older':
                tf.float64,
            'gdp_usd':
                tf.float64,
            'gdp_per_capita_usd':
                tf.float64,
            'openstreetmap_id':
                tf.string,
            'latitude':
                tf.float64,
            'longitude':
                tf.float64,
            'area_sq_km':
                tf.float64,
            'smoking_prevalence':
                tf.float64,
            'diabetes_prevalence':
                tf.float64,
            'infant_mortality_rate':
                tf.float64,
            'nurses_per_1000':
                tf.float64,
            'physicians_per_1000':
                tf.float64,
            'health_expenditure_usd':
                tf.float64,
            'out_of_pocket_health_expenditure_usd':
                tf.float64,
            'school_closing':
                tf.float64,
            'workplace_closing':
                tf.float64,
            'cancel_public_events':
                tf.float64,
            'restrictions_on_gatherings':
                tf.float64,
            'public_transport_closing':
                tf.float64,
            'stay_at_home_requirements':
                tf.float64,
            'restrictions_on_internal_movement':
                tf.float64,
            'international_travel_controls':
                tf.float64,
            'income_support':
                tf.float64,
            'debt_relief':
                tf.float64,
            'fiscal_measures':
                tf.float64,
            'international_support':
                tf.float64,
            'public_information_campaigns':
                tf.float64,
            'testing_policy':
                tf.float64,
            'contact_tracing':
                tf.float64,
            'emergency_investment_in_healthcare':
                tf.float64,
            'investment_in_vaccines':
                tf.float64,
            'facial_coverings':
                tf.float64,
            'vaccination_policy':
                tf.float64,
            'stringency_index':
                tf.float64,
            'average_temperature_celsius':
                tf.float64,
            'minimum_temperature_celsius':
                tf.float64,
            'maximum_temperature_celsius':
                tf.float64,
            'rainfall_mm':
                tf.float64,
            'snowfall_mm':
                tf.float64,
            'dew_point':
                tf.float64,
            'relative_humidity':
                tf.float64,
            'new_tested':
                tf.float64,
            'population_largest_city':
                tf.float64,
            'population_clustered':
                tf.float64,
            'human_capital_index':
                tf.float64,
            'area_rural_sq_km':
                tf.float64,
            'area_urban_sq_km':
                tf.float64,
            'life_expectancy':
                tf.float64,
            'adult_male_mortality_rate':
                tf.float64,
            'adult_female_mortality_rate':
                tf.float64,
            'pollution_mortality_rate':
                tf.float64,
            'comorbidity_mortality_rate':
                tf.float64,
            'mobility_retail_and_recreation':
                tf.float64,
            'mobility_grocery_and_pharmacy':
                tf.float64,
            'mobility_parks':
                tf.float64,
            'mobility_transit_stations':
                tf.float64,
            'mobility_workplaces':
                tf.float64,
            'mobility_residential':
                tf.float64,
            'hospital_beds_per_1000':
                tf.float64,
            'subregion1_code':
                tf.string,
            'subregion1_name':
                tf.string,
            'new_recovered':
                tf.float64,
            'cumulative_recovered':
                tf.float64,
            'elevation_m':
                tf.float64,
            'new_hospitalized_patients':
                tf.float64,
            'cumulative_hospitalized_patients':
                tf.float64,
            'new_intensive_care_patients':
                tf.float64,
            'cumulative_intensive_care_patients':
                tf.float64,
            'new_confirmed_age_0':
                tf.float64,
            'new_confirmed_age_1':
                tf.float64,
            'new_confirmed_age_2':
                tf.float64,
            'new_confirmed_age_3':
                tf.float64,
            'new_confirmed_age_4':
                tf.float64,
            'new_confirmed_age_5':
                tf.float64,
            'new_confirmed_age_6':
                tf.float64,
            'new_confirmed_age_7':
                tf.float64,
            'new_confirmed_age_8':
                tf.float64,
            'new_confirmed_age_9':
                tf.float64,
            'cumulative_confirmed_age_0':
                tf.float64,
            'cumulative_confirmed_age_1':
                tf.float64,
            'cumulative_confirmed_age_2':
                tf.float64,
            'cumulative_confirmed_age_3':
                tf.float64,
            'cumulative_confirmed_age_4':
                tf.float64,
            'cumulative_confirmed_age_5':
                tf.float64,
            'cumulative_confirmed_age_6':
                tf.float64,
            'cumulative_confirmed_age_7':
                tf.float64,
            'cumulative_confirmed_age_8':
                tf.float64,
            'cumulative_confirmed_age_9':
                tf.float64,
            'new_deceased_age_0':
                tf.float64,
            'new_deceased_age_1':
                tf.float64,
            'new_deceased_age_2':
                tf.float64,
            'new_deceased_age_3':
                tf.float64,
            'new_deceased_age_4':
                tf.float64,
            'new_deceased_age_5':
                tf.float64,
            'new_deceased_age_6':
                tf.float64,
            'new_deceased_age_7':
                tf.float64,
            'new_deceased_age_8':
                tf.float64,
            'new_deceased_age_9':
                tf.float64,
            'cumulative_deceased_age_0':
                tf.float64,
            'cumulative_deceased_age_1':
                tf.float64,
            'cumulative_deceased_age_2':
                tf.float64,
            'cumulative_deceased_age_3':
                tf.float64,
            'cumulative_deceased_age_4':
                tf.float64,
            'cumulative_deceased_age_5':
                tf.float64,
            'cumulative_deceased_age_6':
                tf.float64,
            'cumulative_deceased_age_7':
                tf.float64,
            'cumulative_deceased_age_8':
                tf.float64,
            'cumulative_deceased_age_9':
                tf.float64,
            'new_tested_age_0':
                tf.float64,
            'new_tested_age_1':
                tf.float64,
            'new_tested_age_2':
                tf.float64,
            'new_tested_age_3':
                tf.float64,
            'new_tested_age_4':
                tf.float64,
            'new_tested_age_5':
                tf.float64,
            'new_tested_age_6':
                tf.float64,
            'new_tested_age_7':
                tf.float64,
            'new_tested_age_8':
                tf.float64,
            'new_tested_age_9':
                tf.float64,
            'cumulative_tested_age_0':
                tf.float64,
            'cumulative_tested_age_1':
                tf.float64,
            'cumulative_tested_age_2':
                tf.float64,
            'cumulative_tested_age_3':
                tf.float64,
            'cumulative_tested_age_4':
                tf.float64,
            'cumulative_tested_age_5':
                tf.float64,
            'cumulative_tested_age_6':
                tf.float64,
            'cumulative_tested_age_7':
                tf.float64,
            'cumulative_tested_age_8':
                tf.float64,
            'cumulative_tested_age_9':
                tf.float64,
            'new_hospitalized_patients_age_0':
                tf.float64,
            'new_hospitalized_patients_age_1':
                tf.float64,
            'new_hospitalized_patients_age_2':
                tf.float64,
            'new_hospitalized_patients_age_3':
                tf.float64,
            'new_hospitalized_patients_age_4':
                tf.float64,
            'new_hospitalized_patients_age_5':
                tf.float64,
            'new_hospitalized_patients_age_6':
                tf.float64,
            'new_hospitalized_patients_age_7':
                tf.float64,
            'new_hospitalized_patients_age_8':
                tf.float64,
            'new_hospitalized_patients_age_9':
                tf.float64,
            'cumulative_hospitalized_patients_age_0':
                tf.float64,
            'cumulative_hospitalized_patients_age_1':
                tf.float64,
            'cumulative_hospitalized_patients_age_2':
                tf.float64,
            'cumulative_hospitalized_patients_age_3':
                tf.float64,
            'cumulative_hospitalized_patients_age_4':
                tf.float64,
            'cumulative_hospitalized_patients_age_5':
                tf.float64,
            'cumulative_hospitalized_patients_age_6':
                tf.float64,
            'cumulative_hospitalized_patients_age_7':
                tf.float64,
            'cumulative_hospitalized_patients_age_8':
                tf.float64,
            'cumulative_hospitalized_patients_age_9':
                tf.float64,
            'new_intensive_care_patients_age_0':
                tf.float64,
            'new_intensive_care_patients_age_1':
                tf.float64,
            'new_intensive_care_patients_age_2':
                tf.float64,
            'new_intensive_care_patients_age_3':
                tf.float64,
            'new_intensive_care_patients_age_4':
                tf.float64,
            'new_intensive_care_patients_age_5':
                tf.float64,
            'new_intensive_care_patients_age_6':
                tf.float64,
            'new_intensive_care_patients_age_7':
                tf.float64,
            'new_intensive_care_patients_age_8':
                tf.float64,
            'new_intensive_care_patients_age_9':
                tf.float64,
            'cumulative_intensive_care_patients_age_0':
                tf.float64,
            'cumulative_intensive_care_patients_age_1':
                tf.float64,
            'cumulative_intensive_care_patients_age_2':
                tf.float64,
            'cumulative_intensive_care_patients_age_3':
                tf.float64,
            'cumulative_intensive_care_patients_age_4':
                tf.float64,
            'cumulative_intensive_care_patients_age_5':
                tf.float64,
            'cumulative_intensive_care_patients_age_6':
                tf.float64,
            'cumulative_intensive_care_patients_age_7':
                tf.float64,
            'cumulative_intensive_care_patients_age_8':
                tf.float64,
            'cumulative_intensive_care_patients_age_9':
                tf.float64,
            # Stored as string because values look like "30-39"
            'age_bin_0':
                tf.string,
            'age_bin_1':
                tf.string,
            'age_bin_2':
                tf.string,
            'age_bin_3':
                tf.string,
            'age_bin_4':
                tf.string,
            'age_bin_5':
                tf.string,
            'age_bin_6':
                tf.string,
            'age_bin_7':
                tf.string,
            'age_bin_8':
                tf.string,
            'age_bin_9':
                tf.string,
            'new_confirmed_male':
                tf.float64,
            'new_confirmed_female':
                tf.float64,
            'cumulative_confirmed_male':
                tf.float64,
            'cumulative_confirmed_female':
                tf.float64,
            'new_deceased_male':
                tf.float64,
            'new_deceased_female':
                tf.float64,
            'cumulative_deceased_male':
                tf.float64,
            'cumulative_deceased_female':
                tf.float64,
            'new_tested_male':
                tf.float64,
            'new_tested_female':
                tf.float64,
            'cumulative_tested_male':
                tf.float64,
            'cumulative_tested_female':
                tf.float64,
            'new_hospitalized_patients_male':
                tf.float64,
            'new_hospitalized_patients_female':
                tf.float64,
            'cumulative_hospitalized_patients_male':
                tf.float64,
            'cumulative_hospitalized_patients_female':
                tf.float64,
            'new_intensive_care_patients_male':
                tf.float64,
            'new_intensive_care_patients_female':
                tf.float64,
            'cumulative_intensive_care_patients_male':
                tf.float64,
            'cumulative_intensive_care_patients_female':
                tf.float64,
            'subregion2_code':
                tf.string,
            'subregion2_name':
                tf.string,
            'current_hospitalized_patients':
                tf.float64,
            'current_intensive_care_patients':
                tf.float64,
            'current_ventilator_patients':
                tf.float64,
            'search_trends_abdominal_obesity':
                tf.float64,
            'search_trends_abdominal_pain':
                tf.float64,
            'search_trends_acne':
                tf.float64,
            'search_trends_actinic_keratosis':
                tf.float64,
            'search_trends_acute_bronchitis':
                tf.float64,
            'search_trends_adrenal_crisis':
                tf.float64,
            'search_trends_ageusia':
                tf.float64,
            'search_trends_alcoholism':
                tf.float64,
            'search_trends_allergic_conjunctivitis':
                tf.float64,
            'search_trends_allergy':
                tf.float64,
            'search_trends_amblyopia':
                tf.float64,
            'search_trends_amenorrhea':
                tf.float64,
            'search_trends_amnesia':
                tf.float64,
            'search_trends_anal_fissure':
                tf.float64,
            'search_trends_anaphylaxis':
                tf.float64,
            'search_trends_anemia':
                tf.float64,
            'search_trends_angina_pectoris':
                tf.float64,
            'search_trends_angioedema':
                tf.float64,
            'search_trends_angular_cheilitis':
                tf.float64,
            'search_trends_anosmia':
                tf.float64,
            'search_trends_anxiety':
                tf.float64,
            'search_trends_aphasia':
                tf.float64,
            'search_trends_aphonia':
                tf.float64,
            'search_trends_apnea':
                tf.float64,
            'search_trends_arthralgia':
                tf.float64,
            'search_trends_arthritis':
                tf.float64,
            'search_trends_ascites':
                tf.float64,
            'search_trends_asperger_syndrome':
                tf.float64,
            'search_trends_asphyxia':
                tf.float64,
            'search_trends_asthma':
                tf.float64,
            'search_trends_astigmatism':
                tf.float64,
            'search_trends_ataxia':
                tf.float64,
            'search_trends_atheroma':
                tf.float64,
            'search_trends_attention_deficit_hyperactivity_disorder':
                tf.float64,
            'search_trends_auditory_hallucination':
                tf.float64,
            'search_trends_autoimmune_disease':
                tf.float64,
            'search_trends_avoidant_personality_disorder':
                tf.float64,
            'search_trends_back_pain':
                tf.float64,
            'search_trends_bacterial_vaginosis':
                tf.float64,
            'search_trends_balance_disorder':
                tf.float64,
            'search_trends_beaus_lines':
                tf.float64,
            'search_trends_bells_palsy':
                tf.float64,
            'search_trends_biliary_colic':
                tf.float64,
            'search_trends_binge_eating':
                tf.float64,
            'search_trends_bleeding':
                tf.float64,
            'search_trends_bleeding_on_probing':
                tf.float64,
            'search_trends_blepharospasm':
                tf.float64,
            'search_trends_bloating':
                tf.float64,
            'search_trends_blood_in_stool':
                tf.float64,
            'search_trends_blurred_vision':
                tf.float64,
            'search_trends_blushing':
                tf.float64,
            'search_trends_boil':
                tf.float64,
            'search_trends_bone_fracture':
                tf.float64,
            'search_trends_bone_tumor':
                tf.float64,
            'search_trends_bowel_obstruction':
                tf.float64,
            'search_trends_bradycardia':
                tf.float64,
            'search_trends_braxton_hicks_contractions':
                tf.float64,
            'search_trends_breakthrough_bleeding':
                tf.float64,
            'search_trends_breast_pain':
                tf.float64,
            'search_trends_bronchitis':
                tf.float64,
            'search_trends_bruise':
                tf.float64,
            'search_trends_bruxism':
                tf.float64,
            'search_trends_bunion':
                tf.float64,
            'search_trends_burn':
                tf.float64,
            'search_trends_burning_chest_pain':
                tf.float64,
            'search_trends_burning_mouth_syndrome':
                tf.float64,
            'search_trends_candidiasis':
                tf.float64,
            'search_trends_canker_sore':
                tf.float64,
            'search_trends_cardiac_arrest':
                tf.float64,
            'search_trends_carpal_tunnel_syndrome':
                tf.float64,
            'search_trends_cataplexy':
                tf.float64,
            'search_trends_cataract':
                tf.float64,
            'search_trends_chancre':
                tf.float64,
            'search_trends_cheilitis':
                tf.float64,
            'search_trends_chest_pain':
                tf.float64,
            'search_trends_chills':
                tf.float64,
            'search_trends_chorea':
                tf.float64,
            'search_trends_chronic_pain':
                tf.float64,
            'search_trends_cirrhosis':
                tf.float64,
            'search_trends_cleft_lip_and_cleft_palate':
                tf.float64,
            'search_trends_clouding_of_consciousness':
                tf.float64,
            'search_trends_cluster_headache':
                tf.float64,
            'search_trends_colitis':
                tf.float64,
            'search_trends_coma':
                tf.float64,
            'search_trends_common_cold':
                tf.float64,
            'search_trends_compulsive_behavior':
                tf.float64,
            'search_trends_compulsive_hoarding':
                tf.float64,
            'search_trends_confusion':
                tf.float64,
            'search_trends_congenital_heart_defect':
                tf.float64,
            'search_trends_conjunctivitis':
                tf.float64,
            'search_trends_constipation':
                tf.float64,
            'search_trends_convulsion':
                tf.float64,
            'search_trends_cough':
                tf.float64,
            'search_trends_crackles':
                tf.float64,
            'search_trends_cramp':
                tf.float64,
            'search_trends_crepitus':
                tf.float64,
            'search_trends_croup':
                tf.float64,
            'search_trends_cyanosis':
                tf.float64,
            'search_trends_dandruff':
                tf.float64,
            'search_trends_delayed_onset_muscle_soreness':
                tf.float64,
            'search_trends_dementia':
                tf.float64,
            'search_trends_dentin_hypersensitivity':
                tf.float64,
            'search_trends_depersonalization':
                tf.float64,
            'search_trends_depression':
                tf.float64,
            'search_trends_dermatitis':
                tf.float64,
            'search_trends_desquamation':
                tf.float64,
            'search_trends_developmental_disability':
                tf.float64,
            'search_trends_diabetes':
                tf.float64,
            'search_trends_diabetic_ketoacidosis':
                tf.float64,
            'search_trends_diarrhea':
                tf.float64,
            'search_trends_dizziness':
                tf.float64,
            'search_trends_dry_eye_syndrome':
                tf.float64,
            'search_trends_dysautonomia':
                tf.float64,
            'search_trends_dysgeusia':
                tf.float64,
            'search_trends_dysmenorrhea':
                tf.float64,
            'search_trends_dyspareunia':
                tf.float64,
            'search_trends_dysphagia':
                tf.float64,
            'search_trends_dysphoria':
                tf.float64,
            'search_trends_dystonia':
                tf.float64,
            'search_trends_dysuria':
                tf.float64,
            'search_trends_ear_pain':
                tf.float64,
            'search_trends_eczema':
                tf.float64,
            'search_trends_edema':
                tf.float64,
            'search_trends_encephalitis':
                tf.float64,
            'search_trends_encephalopathy':
                tf.float64,
            'search_trends_epidermoid_cyst':
                tf.float64,
            'search_trends_epilepsy':
                tf.float64,
            'search_trends_epiphora':
                tf.float64,
            'search_trends_erectile_dysfunction':
                tf.float64,
            'search_trends_erythema':
                tf.float64,
            'search_trends_erythema_chronicum_migrans':
                tf.float64,
            'search_trends_esophagitis':
                tf.float64,
            'search_trends_excessive_daytime_sleepiness':
                tf.float64,
            'search_trends_eye_pain':
                tf.float64,
            'search_trends_eye_strain':
                tf.float64,
            'search_trends_facial_nerve_paralysis':
                tf.float64,
            'search_trends_facial_swelling':
                tf.float64,
            'search_trends_fasciculation':
                tf.float64,
            'search_trends_fatigue':
                tf.float64,
            'search_trends_fatty_liver_disease':
                tf.float64,
            'search_trends_fecal_incontinence':
                tf.float64,
            'search_trends_fever':
                tf.float64,
            'search_trends_fibrillation':
                tf.float64,
            'search_trends_fibrocystic_breast_changes':
                tf.float64,
            'search_trends_fibromyalgia':
                tf.float64,
            'search_trends_flatulence':
                tf.float64,
            'search_trends_floater':
                tf.float64,
            'search_trends_focal_seizure':
                tf.float64,
            'search_trends_folate_deficiency':
                tf.float64,
            'search_trends_food_craving':
                tf.float64,
            'search_trends_food_intolerance':
                tf.float64,
            'search_trends_frequent_urination':
                tf.float64,
            'search_trends_gastroesophageal_reflux_disease':
                tf.float64,
            'search_trends_gastroparesis':
                tf.float64,
            'search_trends_generalized_anxiety_disorder':
                tf.float64,
            'search_trends_genital_wart':
                tf.float64,
            'search_trends_gingival_recession':
                tf.float64,
            'search_trends_gingivitis':
                tf.float64,
            'search_trends_globus_pharyngis':
                tf.float64,
            'search_trends_goitre':
                tf.float64,
            'search_trends_gout':
                tf.float64,
            'search_trends_grandiosity':
                tf.float64,
            'search_trends_granuloma':
                tf.float64,
            'search_trends_guilt':
                tf.float64,
            'search_trends_hair_loss':
                tf.float64,
            'search_trends_halitosis':
                tf.float64,
            'search_trends_hay_fever':
                tf.float64,
            'search_trends_headache':
                tf.float64,
            'search_trends_heart_arrhythmia':
                tf.float64,
            'search_trends_heart_murmur':
                tf.float64,
            'search_trends_heartburn':
                tf.float64,
            'search_trends_hematochezia':
                tf.float64,
            'search_trends_hematoma':
                tf.float64,
            'search_trends_hematuria':
                tf.float64,
            'search_trends_hemolysis':
                tf.float64,
            'search_trends_hemoptysis':
                tf.float64,
            'search_trends_hemorrhoids':
                tf.float64,
            'search_trends_hepatic_encephalopathy':
                tf.float64,
            'search_trends_hepatitis':
                tf.float64,
            'search_trends_hepatotoxicity':
                tf.float64,
            'search_trends_hiccup':
                tf.float64,
            'search_trends_hip_pain':
                tf.float64,
            'search_trends_hives':
                tf.float64,
            'search_trends_hot_flash':
                tf.float64,
            'search_trends_hydrocephalus':
                tf.float64,
            'search_trends_hypercalcaemia':
                tf.float64,
            'search_trends_hypercapnia':
                tf.float64,
            'search_trends_hypercholesterolemia':
                tf.float64,
            'search_trends_hyperemesis_gravidarum':
                tf.float64,
            'search_trends_hyperglycemia':
                tf.float64,
            'search_trends_hyperhidrosis':
                tf.float64,
            'search_trends_hyperkalemia':
                tf.float64,
            'search_trends_hyperlipidemia':
                tf.float64,
            'search_trends_hypermobility':
                tf.float64,
            'search_trends_hyperpigmentation':
                tf.float64,
            'search_trends_hypersomnia':
                tf.float64,
            'search_trends_hypertension':
                tf.float64,
            'search_trends_hyperthermia':
                tf.float64,
            'search_trends_hyperthyroidism':
                tf.float64,
            'search_trends_hypertriglyceridemia':
                tf.float64,
            'search_trends_hypertrophy':
                tf.float64,
            'search_trends_hyperventilation':
                tf.float64,
            'search_trends_hypocalcaemia':
                tf.float64,
            'search_trends_hypochondriasis':
                tf.float64,
            'search_trends_hypoglycemia':
                tf.float64,
            'search_trends_hypogonadism':
                tf.float64,
            'search_trends_hypokalemia':
                tf.float64,
            'search_trends_hypomania':
                tf.float64,
            'search_trends_hyponatremia':
                tf.float64,
            'search_trends_hypotension':
                tf.float64,
            'search_trends_hypothyroidism':
                tf.float64,
            'search_trends_hypoxemia':
                tf.float64,
            'search_trends_hypoxia':
                tf.float64,
            'search_trends_impetigo':
                tf.float64,
            'search_trends_implantation_bleeding':
                tf.float64,
            'search_trends_impulsivity':
                tf.float64,
            'search_trends_indigestion':
                tf.float64,
            'search_trends_infection':
                tf.float64,
            'search_trends_inflammation':
                tf.float64,
            'search_trends_inflammatory_bowel_disease':
                tf.float64,
            'search_trends_ingrown_hair':
                tf.float64,
            'search_trends_insomnia':
                tf.float64,
            'search_trends_insulin_resistance':
                tf.float64,
            'search_trends_intermenstrual_bleeding':
                tf.float64,
            'search_trends_intracranial_pressure':
                tf.float64,
            'search_trends_iron_deficiency':
                tf.float64,
            'search_trends_irregular_menstruation':
                tf.float64,
            'search_trends_itch':
                tf.float64,
            'search_trends_jaundice':
                tf.float64,
            'search_trends_kidney_failure':
                tf.float64,
            'search_trends_kidney_stone':
                tf.float64,
            'search_trends_knee_pain':
                tf.float64,
            'search_trends_kyphosis':
                tf.float64,
            'search_trends_lactose_intolerance':
                tf.float64,
            'search_trends_laryngitis':
                tf.float64,
            'search_trends_leg_cramps':
                tf.float64,
            'search_trends_lesion':
                tf.float64,
            'search_trends_leukorrhea':
                tf.float64,
            'search_trends_lightheadedness':
                tf.float64,
            'search_trends_low_back_pain':
                tf.float64,
            'search_trends_low_grade_fever':
                tf.float64,
            'search_trends_lymphedema':
                tf.float64,
            'search_trends_major_depressive_disorder':
                tf.float64,
            'search_trends_malabsorption':
                tf.float64,
            'search_trends_male_infertility':
                tf.float64,
            'search_trends_manic_disorder':
                tf.float64,
            'search_trends_melasma':
                tf.float64,
            'search_trends_melena':
                tf.float64,
            'search_trends_meningitis':
                tf.float64,
            'search_trends_menorrhagia':
                tf.float64,
            'search_trends_middle_back_pain':
                tf.float64,
            'search_trends_migraine':
                tf.float64,
            'search_trends_milium':
                tf.float64,
            'search_trends_mitral_insufficiency':
                tf.float64,
            'search_trends_mood_disorder':
                tf.float64,
            'search_trends_mood_swing':
                tf.float64,
            'search_trends_morning_sickness':
                tf.float64,
            'search_trends_motion_sickness':
                tf.float64,
            'search_trends_mouth_ulcer':
                tf.float64,
            'search_trends_muscle_atrophy':
                tf.float64,
            'search_trends_muscle_weakness':
                tf.float64,
            'search_trends_myalgia':
                tf.float64,
            'search_trends_mydriasis':
                tf.float64,
            'search_trends_myocardial_infarction':
                tf.float64,
            'search_trends_myoclonus':
                tf.float64,
            'search_trends_nasal_congestion':
                tf.float64,
            'search_trends_nasal_polyp':
                tf.float64,
            'search_trends_nausea':
                tf.float64,
            'search_trends_neck_mass':
                tf.float64,
            'search_trends_neck_pain':
                tf.float64,
            'search_trends_neonatal_jaundice':
                tf.float64,
            'search_trends_nerve_injury':
                tf.float64,
            'search_trends_neuralgia':
                tf.float64,
            'search_trends_neutropenia':
                tf.float64,
            'search_trends_night_sweats':
                tf.float64,
            'search_trends_night_terror':
                tf.float64,
            'search_trends_nocturnal_enuresis':
                tf.float64,
            'search_trends_nodule':
                tf.float64,
            'search_trends_nosebleed':
                tf.float64,
            'search_trends_nystagmus':
                tf.float64,
            'search_trends_obesity':
                tf.float64,
            'search_trends_onychorrhexis':
                tf.float64,
            'search_trends_oral_candidiasis':
                tf.float64,
            'search_trends_orthostatic_hypotension':
                tf.float64,
            'search_trends_osteopenia':
                tf.float64,
            'search_trends_osteophyte':
                tf.float64,
            'search_trends_osteoporosis':
                tf.float64,
            'search_trends_otitis':
                tf.float64,
            'search_trends_otitis_externa':
                tf.float64,
            'search_trends_otitis_media':
                tf.float64,
            'search_trends_pain':
                tf.float64,
            'search_trends_palpitations':
                tf.float64,
            'search_trends_pancreatitis':
                tf.float64,
            'search_trends_panic_attack':
                tf.float64,
            'search_trends_papule':
                tf.float64,
            'search_trends_paranoia':
                tf.float64,
            'search_trends_paresthesia':
                tf.float64,
            'search_trends_pelvic_inflammatory_disease':
                tf.float64,
            'search_trends_pericarditis':
                tf.float64,
            'search_trends_periodontal_disease':
                tf.float64,
            'search_trends_periorbital_puffiness':
                tf.float64,
            'search_trends_peripheral_neuropathy':
                tf.float64,
            'search_trends_perspiration':
                tf.float64,
            'search_trends_petechia':
                tf.float64,
            'search_trends_phlegm':
                tf.float64,
            'search_trends_photodermatitis':
                tf.float64,
            'search_trends_photophobia':
                tf.float64,
            'search_trends_photopsia':
                tf.float64,
            'search_trends_pleural_effusion':
                tf.float64,
            'search_trends_pleurisy':
                tf.float64,
            'search_trends_pneumonia':
                tf.float64,
            'search_trends_podalgia':
                tf.float64,
            'search_trends_polycythemia':
                tf.float64,
            'search_trends_polydipsia':
                tf.float64,
            'search_trends_polyneuropathy':
                tf.float64,
            'search_trends_polyuria':
                tf.float64,
            'search_trends_poor_posture':
                tf.float64,
            'search_trends_post_nasal_drip':
                tf.float64,
            'search_trends_postural_orthostatic_tachycardia_syndrome':
                tf.float64,
            'search_trends_prediabetes':
                tf.float64,
            'search_trends_proteinuria':
                tf.float64,
            'search_trends_pruritus_ani':
                tf.float64,
            'search_trends_psychosis':
                tf.float64,
            'search_trends_ptosis':
                tf.float64,
            'search_trends_pulmonary_edema':
                tf.float64,
            'search_trends_pulmonary_hypertension':
                tf.float64,
            'search_trends_purpura':
                tf.float64,
            'search_trends_pus':
                tf.float64,
            'search_trends_pyelonephritis':
                tf.float64,
            'search_trends_radiculopathy':
                tf.float64,
            'search_trends_rectal_pain':
                tf.float64,
            'search_trends_rectal_prolapse':
                tf.float64,
            'search_trends_red_eye':
                tf.float64,
            'search_trends_renal_colic':
                tf.float64,
            'search_trends_restless_legs_syndrome':
                tf.float64,
            'search_trends_rheum':
                tf.float64,
            'search_trends_rhinitis':
                tf.float64,
            'search_trends_rhinorrhea':
                tf.float64,
            'search_trends_rosacea':
                tf.float64,
            'search_trends_round_ligament_pain':
                tf.float64,
            'search_trends_rumination':
                tf.float64,
            'search_trends_scar':
                tf.float64,
            'search_trends_sciatica':
                tf.float64,
            'search_trends_scoliosis':
                tf.float64,
            'search_trends_seborrheic_dermatitis':
                tf.float64,
            'search_trends_self_harm':
                tf.float64,
            'search_trends_sensitivity_to_sound':
                tf.float64,
            'search_trends_sexual_dysfunction':
                tf.float64,
            'search_trends_shallow_breathing':
                tf.float64,
            'search_trends_sharp_pain':
                tf.float64,
            'search_trends_shivering':
                tf.float64,
            'search_trends_shortness_of_breath':
                tf.float64,
            'search_trends_shyness':
                tf.float64,
            'search_trends_sinusitis':
                tf.float64,
            'search_trends_skin_condition':
                tf.float64,
            'search_trends_skin_rash':
                tf.float64,
            'search_trends_skin_tag':
                tf.float64,
            'search_trends_skin_ulcer':
                tf.float64,
            'search_trends_sleep_apnea':
                tf.float64,
            'search_trends_sleep_deprivation':
                tf.float64,
            'search_trends_sleep_disorder':
                tf.float64,
            'search_trends_snoring':
                tf.float64,
            'search_trends_sore_throat':
                tf.float64,
            'search_trends_spasticity':
                tf.float64,
            'search_trends_splenomegaly':
                tf.float64,
            'search_trends_sputum':
                tf.float64,
            'search_trends_stomach_rumble':
                tf.float64,
            'search_trends_strabismus':
                tf.float64,
            'search_trends_stretch_marks':
                tf.float64,
            'search_trends_stridor':
                tf.float64,
            'search_trends_stroke':
                tf.float64,
            'search_trends_stuttering':
                tf.float64,
            'search_trends_subdural_hematoma':
                tf.float64,
            'search_trends_suicidal_ideation':
                tf.float64,
            'search_trends_swelling':
                tf.float64,
            'search_trends_swollen_feet':
                tf.float64,
            'search_trends_swollen_lymph_nodes':
                tf.float64,
            'search_trends_syncope':
                tf.float64,
            'search_trends_tachycardia':
                tf.float64,
            'search_trends_tachypnea':
                tf.float64,
            'search_trends_telangiectasia':
                tf.float64,
            'search_trends_tenderness':
                tf.float64,
            'search_trends_testicular_pain':
                tf.float64,
            'search_trends_throat_irritation':
                tf.float64,
            'search_trends_thrombocytopenia':
                tf.float64,
            'search_trends_thyroid_nodule':
                tf.float64,
            'search_trends_tic':
                tf.float64,
            'search_trends_tinnitus':
                tf.float64,
            'search_trends_tonsillitis':
                tf.float64,
            'search_trends_toothache':
                tf.float64,
            'search_trends_tremor':
                tf.float64,
            'search_trends_trichoptilosis':
                tf.float64,
            'search_trends_tumor':
                tf.float64,
            'search_trends_type_2_diabetes':
                tf.float64,
            'search_trends_unconsciousness':
                tf.float64,
            'search_trends_underweight':
                tf.float64,
            'search_trends_upper_respiratory_tract_infection':
                tf.float64,
            'search_trends_urethritis':
                tf.float64,
            'search_trends_urinary_incontinence':
                tf.float64,
            'search_trends_urinary_tract_infection':
                tf.float64,
            'search_trends_urinary_urgency':
                tf.float64,
            'search_trends_uterine_contraction':
                tf.float64,
            'search_trends_vaginal_bleeding':
                tf.float64,
            'search_trends_vaginal_discharge':
                tf.float64,
            'search_trends_vaginitis':
                tf.float64,
            'search_trends_varicose_veins':
                tf.float64,
            'search_trends_vasculitis':
                tf.float64,
            'search_trends_ventricular_fibrillation':
                tf.float64,
            'search_trends_ventricular_tachycardia':
                tf.float64,
            'search_trends_vertigo':
                tf.float64,
            'search_trends_viral_pneumonia':
                tf.float64,
            'search_trends_visual_acuity':
                tf.float64,
            'search_trends_vomiting':
                tf.float64,
            'search_trends_wart':
                tf.float64,
            'search_trends_water_retention':
                tf.float64,
            'search_trends_weakness':
                tf.float64,
            'search_trends_weight_gain':
                tf.float64,
            'search_trends_wheeze':
                tf.float64,
            'search_trends_xeroderma':
                tf.float64,
            'search_trends_xerostomia':
                tf.float64,
            'search_trends_yawn':
                tf.float64,
            'new_recovered_age_0':
                tf.float64,
            'new_recovered_age_1':
                tf.float64,
            'new_recovered_age_2':
                tf.float64,
            'new_recovered_age_3':
                tf.float64,
            'new_recovered_age_4':
                tf.float64,
            'new_recovered_age_5':
                tf.float64,
            'new_recovered_age_6':
                tf.float64,
            'new_recovered_age_7':
                tf.float64,
            'new_recovered_age_8':
                tf.float64,
            'new_recovered_age_9':
                tf.float64,
            'cumulative_recovered_age_0':
                tf.float64,
            'cumulative_recovered_age_1':
                tf.float64,
            'cumulative_recovered_age_2':
                tf.float64,
            'cumulative_recovered_age_3':
                tf.float64,
            'cumulative_recovered_age_4':
                tf.float64,
            'cumulative_recovered_age_5':
                tf.float64,
            'cumulative_recovered_age_6':
                tf.float64,
            'cumulative_recovered_age_7':
                tf.float64,
            'cumulative_recovered_age_8':
                tf.float64,
            'cumulative_recovered_age_9':
                tf.float64,
            'new_recovered_male':
                tf.float64,
            'new_recovered_female':
                tf.float64,
            'cumulative_recovered_male':
                tf.float64,
            'cumulative_recovered_female':
                tf.float64,
            'locality_code':
                tf.string,
            'locality_name':
                tf.string,
            'new_ventilator_patients':
                tf.float64,
            'cumulative_ventilator_patients':
                tf.float64,
            'new_persons_fully_vaccinated_pfizer':
                tf.float64,
            'cumulative_persons_fully_vaccinated_pfizer':
                tf.float64,
            'new_vaccine_doses_administered_pfizer':
                tf.float64,
            'cumulative_vaccine_doses_administered_pfizer':
                tf.float64,
            'new_persons_fully_vaccinated_moderna':
                tf.float64,
            'cumulative_persons_fully_vaccinated_moderna':
                tf.float64,
            'new_vaccine_doses_administered_moderna':
                tf.float64,
            'cumulative_vaccine_doses_administered_moderna':
                tf.float64,
            'new_persons_fully_vaccinated_janssen':
                tf.float64,
            'cumulative_persons_fully_vaccinated_janssen':
                tf.float64,
            'new_vaccine_doses_administered_janssen':
                tf.float64,
            'cumulative_vaccine_doses_administered_janssen':
                tf.float64,
        }),
        supervised_keys=None,
        homepage='https://github.com/GoogleCloudPlatform/covid-19-open-data',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    archive_path = dl_manager.download(
        'https://storage.googleapis.com/covid19-open-data/v3/aggregated.csv.gz?generation=1620814656792419'
    )
    return {tfds.Split.TRAIN: self._generate_examples(dl_manager, archive_path)}

  def _generate_examples(self, dl_manager: tfds.download.DownloadManager,
                         archive_path):
    """Yields examples."""
    pd = tfds.core.lazy_imports.pandas
    beam = tfds.core.lazy_imports.apache_beam

    def cast_according_to_column(feature_type, v):
      if feature_type == tf.string and isinstance(v, (float, int)):
        return str(v)
      return v

    file_handles = dl_manager.iter_archive(archive_path)
    _, file = next(file_handles)

    columns = pd.read_csv(file, nrows=1).columns

    def load_shard(index: int):
      # There is only one file so by using the for we guarantee that the file
      # will be closed.
      for _, file in dl_manager.iter_archive(archive_path):
        df = pd.read_csv(file, skiprows=index, nrows=_BATCH_SIZE)
        features = self.info.features
        result = []
        for i, row in df.iterrows():
          example = {
              k: cast_according_to_column(features[k].dtype, v)
              for k, v in zip(columns, row.values)
          }
          result.append((index + i, example))
        return result

    return (beam.Create(list(range(0, _N_RECORDS, _BATCH_SIZE)))
            | beam.FlatMap(load_shard))



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for file names."""

import dataclasses
import os
import re
import textwrap
from typing import Any, Dict, List, Optional, Tuple, Union

from tensorflow_datasets.core.utils import py_utils

_first_cap_re = re.compile('(.)([A-Z][a-z0-9]+)')
_all_cap_re = re.compile('([a-z0-9])([A-Z])')

_NAME_CLASS = r'[a-zA-Z][\w]*'
_NAME_CLASS_REG = re.compile(r'^' + _NAME_CLASS + r'$')

# Regex matching 'dataset/config:1.*.*/arg=123'
_NAME_REG = re.compile(r'^'
                       r'(?P<dataset_name>([\w\-]+:)?' + _NAME_CLASS + r')'
                       r'(/(?P<config>[\w\-\.]+))?'
                       r'(:(?P<version>(\d+|\*)(\.(\d+|\*)){2}))?'
                       r'(/(?P<kwargs>(\w+=\w+)(,\w+=[^,]+)*))?'
                       r'$')

Value = Union[str, int, float, bool]


@dataclasses.dataclass(eq=True, order=True, frozen=True)
class DatasetName:
  """Dataset namespace+name."""
  namespace: Optional[str]
  name: str

  def __init__(
      self,
      namespace_name: Optional[str] = None,
      # TODO(py3.8): Positional-only arg /,
      *,
      namespace: Optional[str] = None,
      name: Optional[str] = None,
  ):
    if namespace_name and bool(namespace or name):
      raise ValueError(
          "Name should be defined by `DatasetName('ns:name')` or "
          "`DatasetName(namespace='ns', name='name'). Mixing args and kwargs "
          'is invalid.')
    if namespace_name:
      if ':' in namespace_name:
        namespace, name = namespace_name.split(':')
      else:
        namespace, name = None, namespace_name
    super().__setattr__('namespace', namespace)
    super().__setattr__('name', name)

  def __str__(self) -> str:
    return f'{self.namespace}:{self.name}' if self.namespace else self.name

  def __repr__(self) -> str:
    return f'{type(self).__name__}({str(self)!r})'


def is_valid_dataset_name(name_str: str) -> bool:
  """Returns True is the given name is a valid dataset name."""
  res = _NAME_REG.match(name_str)
  return bool(res)


def is_valid_dataset_and_class_name(name_str: str) -> bool:
  """Returns True is the given name is a valid dataset and class name."""
  res = _NAME_CLASS_REG.match(name_str)
  return bool(res)


def parse_builder_name_kwargs(
    name: str,
    **builder_kwargs: Any,
) -> Tuple[DatasetName, Dict[str, Any]]:
  """Normalize builder kwargs.

  Example:

  ```python
  ds_name, builder_kwargs = parse_builder_name_kwargs(
      'kaggle:ds/cfg:1.2.3', data_dir='...'
  )
  ds_name.namespace == 'kaggle'
  ds_name.name == 'ds'
  builder_kwargs == {'config': 'cfg', 'version': '1.2.3', 'data_dir': '...'}
  ```

  Args:
    name: Name
    **builder_kwargs: Builder kwargs

  Returns:
    ns_name: Dataset namespace, or None
    ds_name: Dataset name
    builder_kwargs: Builder kwargs (version, config, data_dir,...)
  """
  name, parsed_builder_kwargs = _dataset_name_and_kwargs_from_name_str(name)
  builder_kwargs = dict(**parsed_builder_kwargs, **builder_kwargs)
  return DatasetName(name), builder_kwargs


def _dataset_name_and_kwargs_from_name_str(
    name_str: str,) -> Tuple[str, Dict[str, Value]]:
  """Extract kwargs from name str."""
  err_msg = textwrap.dedent(f"""\
      Parsing builder name string {name_str} failed.
      The builder name string must be of the following format:
        dataset_name[/config_name][:version][/kwargs]

        Where:

          * dataset_name and config_name are string following python variable naming.
          * version is of the form x.y.z where {{x,y,z}} can be any digit or *.
          * kwargs is a comma list separated of arguments and values to pass to
            builder.

        Examples:
          my_dataset
          my_dataset:1.2.*
          my_dataset/config1
          my_dataset/config1:1.*.*
          my_dataset/config1/arg1=val1,arg2=val2
          my_dataset/config1:1.2.3/right=True,foo=bar,rate=1.2
      """)

  res = _NAME_REG.match(name_str)
  if not res:
    raise ValueError(err_msg)
  name = res.group('dataset_name')
  # Normalize the name to accept CamelCase
  name = camelcase_to_snakecase(name)
  kwargs = _kwargs_str_to_kwargs(res.group('kwargs'))
  try:
    for attr in ['config', 'version']:
      val = res.group(attr)
      if val is None:
        continue
      if attr in kwargs:
        raise ValueError('Dataset %s: cannot pass %s twice.' % (name, attr))
      kwargs[attr] = val
    return name, kwargs
  except Exception as e:  # pylint: disable=broad-except
    py_utils.reraise(e, prefix=err_msg)  # pytype: disable=bad-return-type


def _kwargs_str_to_kwargs(kwargs_str: str):
  """Converts given `kwargs` as str into kwargs dict."""
  if not kwargs_str:
    return {}
  kwarg_strs = kwargs_str.split(',')
  kwargs = {}
  for kwarg_str in kwarg_strs:
    kwarg_name, kwarg_val = kwarg_str.split('=')
    kwargs[kwarg_name] = _cast_to_pod(kwarg_val)
  return kwargs


def _cast_to_pod(val: str) -> Value:
  """Try cast to bool, int, float, str, in that order."""
  bools = {'True': True, 'False': False}
  if val in bools:
    return bools[val]
  try:
    return int(val)
  except ValueError:
    try:
      return float(val)
    except ValueError:
      return val


def camelcase_to_snakecase(name: str) -> str:
  """Convert camel-case string to snake-case."""
  s1 = _first_cap_re.sub(r'\1_\2', name)
  return _all_cap_re.sub(r'\1_\2', s1).lower()


def snake_to_camelcase(name: str) -> str:
  """Convert snake-case string to camel-case string."""
  return ''.join(n.capitalize() for n in name.split('_'))


def filename_prefix_for_name(name: str) -> str:
  if os.path.basename(name) != name:
    raise ValueError('Should be a dataset name, not a path: %s' % name)
  return camelcase_to_snakecase(name)


def filename_prefix_for_split(name: str, split: str) -> str:
  if os.path.basename(name) != name:
    raise ValueError('Should be a dataset name, not a path: %s' % name)
  return '%s-%s' % (filename_prefix_for_name(name), split)


def sharded_filenames(filename_prefix: str, num_shards: int) -> List[str]:
  """Sharded filenames given prefix and number of shards."""
  shard_suffix = '%05d-of-%05d'
  return [
      '%s-%s' % (filename_prefix, shard_suffix % (i, num_shards))
      for i in range(num_shards)
  ]


def filepattern_for_dataset_split(dataset_name: str,
                                  split: str,
                                  data_dir: str,
                                  filetype_suffix: Optional[str] = None,
                                  num_shards: Optional[int] = None) -> str:
  """Returns the file pattern for the given dataset.

  Args:
    dataset_name: Name of the dataset
    split: Name of the requested split
    data_dir: The base folder that contains the dataset.
    filetype_suffix: Optional suffix, e.g. tfrecord
    num_shards: Optional argument. If specified, will return file@num_shards
      notation, otherwise file*.
  """
  prefix = filename_prefix_for_split(dataset_name, split)
  if filetype_suffix:
    prefix += '.%s' % filetype_suffix
  filepath = os.path.join(os.fspath(data_dir), prefix)
  if num_shards:
    return '%s@%d' % (filepath, num_shards)
  return '%s*' % filepath


def filenames_for_dataset_split(
    dataset_name: str,
    split: str,
    num_shards: int,
    filetype_suffix: Optional[str] = None) -> List[str]:
  prefix = filename_prefix_for_split(dataset_name, split)
  if filetype_suffix:
    prefix += '.%s' % filetype_suffix
  return sharded_filenames(prefix, num_shards)


def filepaths_for_dataset_split(
    dataset_name: str,
    split: str,
    num_shards: int,
    data_dir: str,
    filetype_suffix: Optional[str] = None) -> List[str]:
  """File paths of a given dataset split."""
  filenames = filenames_for_dataset_split(
      dataset_name=dataset_name,
      split=split,
      num_shards=num_shards,
      filetype_suffix=filetype_suffix,
  )
  filepaths = [os.path.join(data_dir, fname) for fname in filenames]
  return filepaths


@dataclasses.dataclass(eq=True, frozen=True)
class FilenameInfo:
  """Structure representing a filename.

  Filenames have the following specs:

  ```
  <dataset_name>-<split_name>.<file-extension>-xxxxxx-of-yyyyyy
  ```

  """
  dataset_name: str
  split: str
  filetype_suffix: str
  shard_index: int
  num_shards: int

  @classmethod
  def from_str(cls, filename: str) -> 'FilenameInfo':
    """Factory to create a `FilenameInfo` from filename."""
    match = _parse_filename(filename)
    if not match:  # No match found
      raise ValueError(
          f'Filename {filename!r} does not follow pattern: '
          '<dataset_name>-<split_name>.<file-extension>-xxxxxx-of-yyyyyy')
    values = match.groupdict()
    return cls(
        dataset_name=values['dataset_name'],
        split=values['split'],
        filetype_suffix=values['filetype_suffix'],
        shard_index=int(values['shard_index']),
        num_shards=int(values['num_shards']),
    )

  @staticmethod
  def is_valid(filename: str) -> bool:
    """Returns True if the filename follow the given pattern."""
    return bool(_parse_filename(filename))

  def __str__(self) -> str:
    # Note: It's possible for `shard_index` and `num_shards` to exceed 5 digits,
    # e.g., "000123-of-654321", "123456-of-654321".
    num_digits = max(5, len(str(self.num_shards)))
    return (
        f'{self.dataset_name}-{self.split}.{self.filetype_suffix}-'
        f'{self.shard_index:0{num_digits}}-of-{self.num_shards:0{num_digits}}')


def _parse_filename(filename: str) -> Optional['re.Match']:  # pytype: disable=module-attr
  """Parse the tf-record filename."""
  pattern = (rf'(?P<dataset_name>{_NAME_CLASS})-(?P<split>\w+)\.'
             r'(?P<filetype_suffix>\w+)-'
             r'(?P<shard_index>\d{5,})-of-(?P<num_shards>\d{5,})')
  return re.fullmatch(pattern, filename)



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tensorflow_datasets.core.features.top_level_feature."""

import tensorflow as tf
from tensorflow_datasets import testing
from tensorflow_datasets.core import features as features_lib
from tensorflow_datasets.core.features import top_level_feature


class FeaturesManagerTest(testing.TestCase):

  def test_sequence_rank(self):

    self.assertEqual(
        1,
        top_level_feature._get_sequence_rank({
            'a':
                features_lib.TensorInfo(
                    shape=(None, 3), dtype=tf.int32, sequence_rank=1),
            'b':
                features_lib.TensorInfo(
                    shape=(None,), dtype=tf.int32, sequence_rank=1),
        }))

    with self.assertRaisesWithPredicateMatch(NotImplementedError,
                                             'mixing sequence and context'):
      top_level_feature._get_sequence_rank({
          'a':
              features_lib.TensorInfo(shape=(), dtype=tf.int32),
          'b':
              features_lib.TensorInfo(
                  shape=(None,), dtype=tf.int32, sequence_rank=1),
      })

  def test_flatten_nested(self):

    f = features_lib.FeaturesDict({
        'a':
            tf.int32,
        'b': {
            'c': {
                'd': tf.int32,
                'e': tf.int32,
            },
        },
        'f':
            features_lib.Sequence({
                'g': features_lib.Sequence(tf.int32),
                'h': tf.int32,
            }),
    })

    flat1 = f._flatten({
        'a': 'a',
        'b': {
            'c': {
                'd': {
                    'd': 123
                },
            },
        },
        'f': {
            'g': 'g',
        },
    })
    self.assertEqual(
        flat1,
        [
            'a',
            {
                'd': 123
            },
            None,  # 'e'
            'g',
            None,  # h
        ])
    self.assertEqual(
        f._nest(flat1), {
            'a': 'a',
            'b': {
                'c': {
                    'd': {
                        'd': 123
                    },
                    'e': None,
                },
            },
            'f': {
                'g': 'g',
                'h': None,
            },
        })

    f = features_lib.FeaturesDict({
        'a': tf.int32,
        'b': {
            'c': tf.int32,
        },
    })
    with self.assertRaisesWithPredicateMatch(ValueError, 'received a non dict'):
      f._flatten({'b': 123})

    with self.assertRaisesWithPredicateMatch(ValueError,
                                             'Unrecognized keys: [\'d\']'):
      f._flatten({'b': {'c': 123, 'd': 123}})

    with self.assertRaisesWithPredicateMatch(
        ValueError, 'Expected length 2 does not match input length 3'):
      f._nest([None, None, None])

  def test_top_level(self):

    f = features_lib.FeaturesDict({
        'a': tf.int32,
        'b': {
            'c': tf.int32,
        },
    })

    # Only top level can be decoded
    f.decode_example({
        'a': 1,
        'b': {
            'c': 2,
        },
    })


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for horses or humans data loading."""

from tensorflow_datasets.image_classification import horses_or_humans
import tensorflow_datasets.testing as tfds_test

horses_or_humans._IMAGE_SHAPE = (None, None, 3)  # pylint: disable=protected-access


class HorsesOrHumansTest(tfds_test.DatasetBuilderTestCase):
  DATASET_CLASS = horses_or_humans.HorsesOrHumans

  SPLITS = {
      'train': 2,
      'test': 2,
  }

  DL_EXTRACT_RESULT = ['hoh_train.zip', 'hoh_test.zip']


if __name__ == '__main__':
  tfds_test.test_main()



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tensorflow_datasets.core.community.register_path."""

import contextlib
import datetime
import json
import os
import pathlib
import sys
import tempfile
import textwrap
from typing import Iterator
from unittest import mock

import pytest

from tensorflow_datasets.core import dataset_builder
from tensorflow_datasets.core import registered
from tensorflow_datasets.core import utils
from tensorflow_datasets.core.community import cache
from tensorflow_datasets.core.community import dataset_sources
from tensorflow_datasets.core.community import register_package


@contextlib.contextmanager
def mock_cache_path(new_cache_dir: utils.PathLike) -> Iterator[None]:
  """Mock which overwrite the cache path."""
  new_dir = utils.as_path(new_cache_dir)

  # Use `__wrapped__` to access the original function wrapped inside
  # `functools.lru_cache`
  new_cache_path = utils.memoize()(cache.cache_path.__wrapped__)
  new_module_path = utils.memoize()(cache.module_path.__wrapped__)
  with mock.patch.object(cache, '_default_cache_dir', return_value=new_dir), \
       mock.patch.object(cache, 'cache_path', new_cache_path), \
       mock.patch.object(cache, 'module_path', new_module_path):
    yield


@pytest.fixture(scope='module')
def dummy_register():
  """Dummy register."""

  with tempfile.TemporaryDirectory() as tmp_path:
    tmp_path = pathlib.Path(tmp_path)

    source_path = utils.tfds_path() / 'testing/dummy_dataset/dummy_dataset.py'

    # Single-file dataset package (without checksums)
    src_single = dataset_sources.DatasetSource.from_json(os.fspath(source_path))

    # Multi-file dataset package (with checksums)
    src_multi = dataset_sources.DatasetSource.from_json({
        'root_path': os.fspath(source_path.parent),
        'filenames': ['checksums.tsv', 'dummy_dataset.py'],
    })
    src_multi_json = json.dumps(src_multi.to_json())  # `dict` -> `str`

    # Create the remote index content
    # Note the absence of `"` for the `src_multi_json` as it is parsed as `dict`
    content = textwrap.dedent(f"""\
        {{"name": "kaggle:dummy_dataset", "source": "{src_single.to_json()}"}}
        {{"name": "kaggle:ds1", "source": "{src_single.to_json()}"}}
        {{"name": "mlds:dummy_dataset", "source": {src_multi_json}}}
        """)
    dummy_path = tmp_path / 'dummy-community-datasets.toml'
    dummy_path.write_text(content)

    with mock_cache_path(tmp_path / 'cache'):
      yield register_package.PackageRegister(path=dummy_path)


def test_builder_cls(dummy_register):  # pylint: disable=redefined-outer-name

  # The dataset will be installed in the cache
  installed_path = cache.cache_path()
  installed_path /= 'modules/tfds_community/kaggle/dummy_dataset'
  assert not installed_path.exists()

  ds_name = utils.DatasetName('kaggle:dummy_dataset')
  builder_cls = dummy_register.builder_cls(ds_name)
  assert builder_cls.name == 'dummy_dataset'

  clshash = 'd56c12548420c4101f36637fea941e4cbfc55aa3a005e0ea6d29b6401ec1e8bd'
  assert installed_path / f'{clshash}/dummy_dataset.py' == builder_cls.code_path
  assert 'kaggle' in builder_cls.code_path.parts
  assert issubclass(builder_cls, dataset_builder.DatasetBuilder)
  assert not builder_cls.url_infos  # No checksums installed with the package

  # Dataset installed in the cache
  # Filename should be deterministic
  assert list(sorted(installed_path.iterdir())) == [installed_path / clshash]

  # Reusing the dataset should re-use the cache
  with mock.patch.object(
      register_package,
      '_download_and_cache',
      side_effect=ValueError('Dataset should have been cached already')):
    ds_name = utils.DatasetName('kaggle:dummy_dataset')
    builder_cls2 = dummy_register.builder_cls(ds_name)
  assert builder_cls is builder_cls2

  # Datasets from different namespace can have the same name
  ds_name = utils.DatasetName('mlds:dummy_dataset')
  builder_cls = dummy_register.builder_cls(ds_name)
  assert 'mlds' in builder_cls.code_path.parts
  assert issubclass(builder_cls, dataset_builder.DatasetBuilder)
  # Checksums have been correctly installed
  assert 'http://dummy.org/data.txt' in builder_cls.url_infos

  with pytest.raises(registered.DatasetNotFoundError):
    dummy_register.builder(utils.DatasetName('other:ds0'))


def test_register_path_list_builders(dummy_register):  # pylint: disable=redefined-outer-name
  assert dummy_register.list_builders() == [
      'kaggle:ds1',
      'kaggle:dummy_dataset',
      'mlds:dummy_dataset',
  ]


def test_dataset_package():
  """Exports/imports operation should be identity."""
  pkg = register_package.DatasetPackage(
      name=utils.DatasetName('ns:ds'),
      source=dataset_sources.DatasetSource.from_json(
          'github://<owner>/<name>/tree/<branch>/my_ds/ds.py',),
  )
  assert register_package.DatasetPackage.from_json(pkg.to_json()) == pkg

  pkg2 = register_package._InstalledPackage(
      package=pkg,
      instalation_date=datetime.datetime.now(),
      hash='asdajhdadsadsad',
  )
  assert register_package._InstalledPackage.from_json(pkg2.to_json()) == pkg2


def test_mock_cache_path(tmp_path: pathlib.Path):
  with mock_cache_path(tmp_path):
    assert os.fspath(tmp_path) not in sys.path
    assert cache.cache_path() == tmp_path
    assert cache.module_path() == tmp_path / 'modules'
    assert os.fspath(tmp_path / 'modules') in sys.path



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module defines the methods a logger implementation should define."""

from typing import Dict, Optional, Union

from tensorflow_datasets.core import tfrecords_reader
from tensorflow_datasets.core.utils import read_config as tfds_read_config


class Logger:
  """Defines interface any TFDS logger must implement.

  Registered loggers methods are called on TFDS events, synchronously, from the
  same thread the call was made, in sequence, and in registration order.
  Exceptions are *NOT* caught.
  """

  def as_dataset(self, *, dataset_name: str, config_name: Optional[str],
                 version: str, data_path: str,
                 split: Union[str, tfrecords_reader.ReadInstruction],
                 batch_size: Optional[int], shuffle_files: bool,
                 read_config: tfds_read_config.ReadConfig, as_supervised: bool,
                 decoders: Dict[str, str]):
    """Callback called when user calls `dataset_builder.as_dataset`.

    Callback is also triggered by `tfds.load`, which calls `as_dataset`.
    The logger MUST NOT mutate passed objects (decoders, read_config, ...).

    Args:
      dataset_name: the name of the dataset. E.g.: "mnist".
      config_name: the name of the config or None.
      version: the dataset version. E.g.: "1.2.3".
      data_path: The path to directory of the dataset loaded. E.g.:
        "/home/alice/.tensorflow_datasets/mnist/1.2.3".
      split: name of the split requested by user. 'all' for all splits.
      batch_size: See DatasetBuilder.as_dataset docstring.
      shuffle_files: -
      read_config: -
      as_supervised: -
      decoders: flatten dict of decoders dict given to `as_dataset`, with the
        values being `{module_name}.{class_name}` instad of a
    """
    raise NotImplementedError



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""wiki_table_questions dataset."""

import textwrap
from unittest import mock

import tensorflow as tf
import tensorflow_datasets.public_api as tfds
from tensorflow_datasets.structured.wiki_table_questions import wiki_table_questions


class WikiTableQuestionsTest(tfds.testing.DatasetBuilderTestCase):
  """Tests for wiki_table_questions dataset."""
  DATASET_CLASS = wiki_table_questions.WikiTableQuestions
  SPLITS = {
      'train': 2,
      'split-1-train': 1,
      'split-1-dev': 1,
      'split-2-train': 1,
      'split-2-dev': 1,
      'split-3-train': 1,
      'split-3-dev': 1,
      'test': 2,
  }

  OVERLAPPING_SPLITS = ['split-{}-train'.format(i) for i in range(1, 4)
                       ] + ['split-{}-dev'.format(i) for i in range(1, 4)]

  def test_generate_examples(self):
    examples_file = textwrap.dedent("""
      id	utterance	context	targetValue
      nu-0	which country had the most cyclists finish within the top 5?	csv/203-csv/733.csv	Italy
      nu-165	who was the first cyclist to finish?	csv/203-csv/733.csv	Alejandro Valverde
    """)
    csv_203_csv_733 = textwrap.dedent("""
      "Rank","Cyclist","Team","Time","UCI ProTour\nPoints"
      "1","Alejandro Valverde (ESP)","Caisse d'Epargne","5h 29' 10\"","40"
      "2","Davide Rebellin (ITA)","Gerolsteiner","s.t.","25"
      "3","Paolo Bettini (ITA)","Quick Step","s.t.","20"
    """)
    table_formated = [{
        'column_header': 'Rank',
        'row_number': 1,
        'content': '1',
    }, {
        'column_header': 'Cyclist',
        'row_number': 1,
        'content': 'Alejandro Valverde (ESP)',
    }, {
        'column_header': 'Team',
        'row_number': 1,
        'content': """Caisse d'Epargne""",
    }, {
        'column_header': 'Time',
        'row_number': 1,
        'content': '''5h 29' 10"''',
    }, {
        'column_header': 'UCI ProTour\nPoints',
        'row_number': 1,
        'content': '40',
    }, {
        'column_header': 'Rank',
        'row_number': 2,
        'content': '2',
    }, {
        'column_header': 'Cyclist',
        'row_number': 2,
        'content': 'Davide Rebellin (ITA)',
    }, {
        'column_header': 'Team',
        'row_number': 2,
        'content': 'Gerolsteiner',
    }, {
        'column_header': 'Time',
        'row_number': 2,
        'content': 's.t.',
    }, {
        'column_header': 'UCI ProTour\nPoints',
        'row_number': 2,
        'content': '25',
    }, {
        'column_header': 'Rank',
        'row_number': 3,
        'content': '3',
    }, {
        'column_header': 'Cyclist',
        'row_number': 3,
        'content': 'Paolo Bettini (ITA)',
    }, {
        'column_header': 'Team',
        'row_number': 3,
        'content': 'Quick Step',
    }, {
        'column_header': 'Time',
        'row_number': 3,
        'content': 's.t.',
    }, {
        'column_header': 'UCI ProTour\nPoints',
        'row_number': 3,
        'content': '20',
    }]
    expected_examples = [{
        'input_text': {
            'table':
                table_formated,
            'context':
                'which country had the most cyclists finish within the top 5?'
        },
        'target_text': 'Italy'
    }, {
        'input_text': {
            'table': table_formated,
            'context': 'who was the first cyclist to finish?'
        },
        'target_text': 'Alejandro Valverde'
    }]
    tf_mock = mock.Mock()
    tf_mock.gfile.GFile.side_effect = lambda f: csv_203_csv_733 if f else examples_file
    dataset = wiki_table_questions.WikiTableQuestions()
    with mock.patch.object(tf, 'io', return_value=tf_mock):
      for i, (_, example) in enumerate(dataset._generate_examples('', '')):
        self.assertCountEqual(example, expected_examples[i])


if __name__ == '__main__':
  tfds.testing.test_main()



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tensorflow_datasets.core.community.register_path."""

import os
import pathlib
import tempfile
import textwrap

import pytest

from tensorflow_datasets import testing
from tensorflow_datasets.core import registered
from tensorflow_datasets.core import utils
from tensorflow_datasets.core.community import register_path


class Ds0(testing.DummyDataset):
  pass


class Ds1(testing.DummyDataset):
  pass


@pytest.fixture(scope='module')
def dummy_register():
  """Dummy register."""
  with tempfile.TemporaryDirectory() as tmp_path:
    tmp_path = pathlib.Path(tmp_path)

    # Prepare the datasets
    # Namespace 0
    Ds0(data_dir=tmp_path / 'kaggle').download_and_prepare()
    Ds1(data_dir=tmp_path / 'kaggle2').download_and_prepare()
    # Namespace 1
    Ds0(data_dir=tmp_path / 'mlds').download_and_prepare()
    # Namespace 2: (non-existing)

    content = textwrap.dedent(f"""
        [Namespaces]
        kaggle=[
            '{os.fspath(tmp_path / 'kaggle')}',
            '{os.fspath(tmp_path / 'kaggle2')}',
        ]
        mlds='{os.fspath(tmp_path / 'mlds')}'
        other='/tmp/path/to/non-existing-path'
        """)

    dummy_path = tmp_path / 'dummy-community-datasets.toml'
    dummy_path.write_text(content)
    yield register_path.DataDirRegister(path=dummy_path)


def test_register_builder(dummy_register):  # pylint: disable=redefined-outer-name
  builder = dummy_register.builder(utils.DatasetName('kaggle:ds0'))
  assert 'kaggle' in builder.data_path.parts

  # Same dataset name can be loaded from different namespace
  builder = dummy_register.builder(utils.DatasetName('mlds:ds0'))
  assert 'mlds' in builder.data_path.parts

  builder = dummy_register.builder(
      utils.DatasetName('mlds:ds0'),
      data_dir=None,  # data_dir can be passed only if None
      version='1.0.0',
  )
  assert 'mlds' in builder.data_path.parts

  with pytest.raises(ValueError, match='`data_dir` cannot be set for'):
    dummy_register.builder(
        utils.DatasetName('mlds:ds0'), data_dir='/path/to/data_dir')

  with pytest.raises(KeyError, match='Namespace .* not found.'):
    dummy_register.builder(utils.DatasetName('non-existing-namespace:ds0'))

  with pytest.raises(registered.DatasetNotFoundError):
    dummy_register.builder(utils.DatasetName('other:ds0'))


def test_register_path_list_builders(dummy_register):  # pylint: disable=redefined-outer-name
  assert dummy_register.list_builders() == [
      'kaggle:ds0',
      'kaggle:ds1',
      'mlds:ds0',
  ]



# coding=utf-8
# Copyright 2021 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Imagenet LT datasets."""

import io
import os

import tensorflow as tf
from tensorflow_datasets.image_classification import imagenet
import tensorflow_datasets.public_api as tfds

_DESCRIPTION = """\
ImageNet-LT is a subset of original ImageNet ILSVRC 2012 dataset.
The training set is subsampled such that the number of images per class
follows a long-tailed distribution. The class with the maximum number of images
contains 1,280 examples, whereas the class with the minumum number of images
contains only 5 examples. The dataset also has a balanced validation set,
which is also a subset of the ImageNet ILSVRC 2012 training set and contains
20 images per class. The test set of this dataset is the same as the validation
set of the original ImageNet ILSVRC 2012 dataset.

The original ImageNet ILSVRC 2012 dataset must be downloaded manually, and
its path should be set with --manual_dir in order to generate this dataset.
"""

_CITATION = r"""\
@inproceedings{openlongtailrecognition,
  title={Large-Scale Long-Tailed Recognition in an Open World},
  author={Liu, Ziwei and Miao, Zhongqi and Zhan, Xiaohang and Wang, Jiayun and Gong, Boqing and Yu, Stella X.},
  booktitle={IEEE Conference on Computer Vision and Pattern Recognition (CVPR)},
  year={2019},
  url={https://github.com/zhmiao/OpenLongTailRecognition-OLTR}
}
"""

_LABELS_FNAME = 'image_classification/imagenet2012_labels.txt'

_TRAIN_SUBSET = ('https://drive.google.com/uc?export=download&'
                 'id=1Sl1cwy6Dei1I1BMS1YKjI35fkaR1UA_7')

_VAL_SUBSET = ('https://drive.google.com/uc?export=download&'
               'id=1AjYczW4khrQrwPIygXUkHF-Qv4QGEsCF')


class ImagenetLt(tfds.core.GeneratorBasedBuilder):
  """Long-tailed version of the ImageNet2012 dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  MANUAL_DOWNLOAD_INSTRUCTIONS = """\
  manual_dir should contain two files: ILSVRC2012_img_train.tar and
  ILSVRC2012_img_val.tar.
  You need to register on http://www.image-net.org/download-images in order
  to get the link to download the dataset.
  """

  def _info(self):
    """Define the dataset info."""
    names_file = tfds.core.tfds_path(_LABELS_FNAME)
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(),
            'label': tfds.features.ClassLabel(names_file=names_file),
            'file_name': tfds.features.Text(),  # Eg: 'n15075141_54.JPEG'
        }),
        supervised_keys=('image', 'label'),
        homepage='https://github.com/zhmiao/OpenLongTailRecognition-OLTR',
        citation=_CITATION)

  def _postprocess_subset_list(self, subset):
    """Postprocess the subset list."""

    # The original subset contains lines such as
    # "train/n01440764/n01440764_190.JPEG 0".
    # Postprocessing removes the redundant info and only keeps the image name,
    # i.e. n01440764_190.JPEG.
    post_subset = [
        os.path.basename(cur_item.split(' ')[0]) for cur_item in subset
    ]
    return post_subset

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""

    train_path = os.path.join(dl_manager.manual_dir, 'ILSVRC2012_img_train.tar')
    val_path = os.path.join(dl_manager.manual_dir, 'ILSVRC2012_img_val.tar')

    # We don't import the original test split, as it doesn't include labels.
    # These were never publicly released.
    if not tf.io.gfile.exists(train_path) or not tf.io.gfile.exists(val_path):
      raise AssertionError(
          'ImageNet-LT requires manual download of the ImageNet2012 data. '
          'Please download the train and val set and place them into:'
          '{}, {}'.format(train_path, val_path))

    # Download and load subset file.
    downloaded_dirs = dl_manager.download({
        'train': _TRAIN_SUBSET,
        'validation': _VAL_SUBSET,
    })

    with tf.io.gfile.GFile(downloaded_dirs['train']) as fp:
      train_subset = fp.read().splitlines()
    train_subset = self._postprocess_subset_list(train_subset)

    with tf.io.gfile.GFile(downloaded_dirs['validation']) as fp:
      val_subset = fp.read().splitlines()
    val_subset = self._postprocess_subset_list(val_subset)

    return {
        'train':
            self._generate_examples(
                archive=dl_manager.iter_archive(train_path),
                subset=train_subset),
        'validation':
            self._generate_examples(
                archive=dl_manager.iter_archive(train_path), subset=val_subset),
        'test':
            self._generate_examples(
                archive=dl_manager.iter_archive(val_path),
                validation_labels=imagenet.get_validation_labels(val_path))
    }

  def _generate_examples(self, archive, subset=None, validation_labels=None):
    """Yields examples."""
    # Test split in ImageNet-LT is the validation split in the original ImageNet
    if validation_labels:
      for key, example in imagenet.generate_examples_validation(
          archive, validation_labels):
        yield key, example

    # Training and validation split. Main archive contains archives names after
    # a synset noun.
    # Each sub-archive contains pictures associated to that synset.
    for fname, fobj in archive:
      label = fname[:-4]  # fname is something like 'n01632458.tar'
      # TODO(b/117643231): in py3, the following lines trigger tarfile module
      # to call `fobj.seekable()`, which Gfile doesn't have. We should find an
      # alternative, as this loads ~150MB in RAM.
      fobj_mem = io.BytesIO(fobj.read())
      for image_fname, image in tfds.download.iter_archive(
          fobj_mem, tfds.download.ExtractMethod.TAR_STREAM):  # pytype: disable=wrong-arg-types  # gen-stub-imports
        if subset is None or image_fname in subset:  # filtering using subset.
          record = {
              'file_name': image_fname,
              'image': image,
              'label': label,
          }
          yield image_fname, record
